@@ -4,7 +4,6 @@
 #include <QtCore/QStringList>
 #include <QtCore/QDebug>
 #include <qmetaobject.h>
-#include <__package_shared.h>
 
 #include <qxmlstream.h>
 #include <QVariant>
@@ -12,6 +11,8 @@
 #include <qvector.h>
 #include <qxmlstream.h>
 
+#include "qtscriptshell_QXmlStreamAttributes.h"
+
 static const char * const qtscript_QXmlStreamAttributes_function_names[] = {
     "QXmlStreamAttributes"
     // static
@@ -33,14 +34,10 @@ static const char * const qtscript_QXmlStreamAttributes_function_names[] = {
     , "isSharedWith"
     , "last"
     , "lastIndexOf"
-    , "length"
     , "mid"
     , "equals"
     , "prepend"
     , "remove"
-    , "removeAt"
-    , "removeFirst"
-    , "removeLast"
     , "replace"
     , "reserve"
     , "resize"
@@ -48,10 +45,6 @@ static const char * const qtscript_QXmlStreamAttributes_function_names[] = {
     , "size"
     , "squeeze"
     , "startsWith"
-    , "swap"
-    , "takeAt"
-    , "takeFirst"
-    , "takeLast"
     , "toList"
     , "value"
     , "toString"
@@ -78,14 +71,10 @@ static const char * const qtscript_QXmlStreamAttributes_function_signatures[] =
     , "List other"
     , ""
     , "QXmlStreamAttribute t, int from"
-    , ""
-    , "int pos, int len"
+    , "int pos, int length"
     , "List v"
     , "QXmlStreamAttribute t"
     , "int i\nint i, int n"
-    , "int i"
-    , ""
-    , ""
     , "int i, QXmlStreamAttribute t"
     , "int size"
     , "int size"
@@ -93,10 +82,6 @@ static const char * const qtscript_QXmlStreamAttributes_function_signatures[] =
     , ""
     , ""
     , "QXmlStreamAttribute t"
-    , "List other"
-    , "int i"
-    , ""
-    , ""
     , ""
     , "String namespaceUri, String name\nString qualifiedName"
 ""
@@ -123,14 +108,10 @@ static const int qtscript_QXmlStreamAttributes_function_lengths[] = {
     , 1
     , 0
     , 2
-    , 0
     , 2
     , 1
     , 1
     , 2
-    , 1
-    , 0
-    , 0
     , 2
     , 1
     , 1
@@ -138,10 +119,6 @@ static const int qtscript_QXmlStreamAttributes_function_lengths[] = {
     , 0
     , 0
     , 1
-    , 1
-    , 1
-    , 0
-    , 0
     , 0
     , 2
     , 0
@@ -160,9 +137,11 @@ static QScriptValue qtscript_QXmlStreamAttributes_throw_ambiguity_error_helper(
 
 Q_DECLARE_METATYPE(QXmlStreamAttributes)
 Q_DECLARE_METATYPE(QXmlStreamAttributes*)
+Q_DECLARE_METATYPE(QtScriptShell_QXmlStreamAttributes)
+Q_DECLARE_METATYPE(QtScriptShell_QXmlStreamAttributes*)
 Q_DECLARE_METATYPE(QXmlStreamAttribute)
-Q_DECLARE_METATYPE(QVector<QXmlStreamAttribute >)
-Q_DECLARE_METATYPE(QList<QXmlStreamAttribute >)
+Q_DECLARE_METATYPE(QVector<QXmlStreamAttribute>)
+Q_DECLARE_METATYPE(QList<QXmlStreamAttribute>)
 Q_DECLARE_METATYPE(QStringRef)
 
 //
@@ -179,7 +158,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     if (context->callee().isFunction())
         _id = context->callee().data().toUInt32();
     else
-        _id = 0xBABE0000 + 37;
+        _id = 0xBABE0000 + 29;
 #endif
     Q_ASSERT((_id & 0xFFFF0000) == 0xBABE0000);
     _id &= 0x0000FFFF;
@@ -215,7 +194,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     case 1:
     if (context->argumentCount() == 1) {
         int _q_arg0 = context->argument(0).toInt32();
-        const QXmlStreamAttribute& _q_result = _q_self->at(_q_arg0);
+        QXmlStreamAttribute _q_result = _q_self->at(_q_arg0);
         return qScriptValueFromValue(context->engine(), _q_result);
     }
     break;
@@ -272,20 +251,20 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     case 8:
     if (context->argumentCount() == 1) {
         QXmlStreamAttribute _q_arg0 = qscriptvalue_cast<QXmlStreamAttribute>(context->argument(0));
-        QVector<QXmlStreamAttribute >& _q_result = _q_self->fill(_q_arg0);
+        QVector<QXmlStreamAttribute> _q_result = _q_self->fill(_q_arg0);
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     if (context->argumentCount() == 2) {
         QXmlStreamAttribute _q_arg0 = qscriptvalue_cast<QXmlStreamAttribute>(context->argument(0));
         int _q_arg1 = context->argument(1).toInt32();
-        QVector<QXmlStreamAttribute >& _q_result = _q_self->fill(_q_arg0, _q_arg1);
+        QVector<QXmlStreamAttribute> _q_result = _q_self->fill(_q_arg0, _q_arg1);
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     break;
 
     case 9:
     if (context->argumentCount() == 0) {
-        const QXmlStreamAttribute& _q_result = _q_self->first();
+        QXmlStreamAttribute _q_result = _q_self->first();
         return qScriptValueFromValue(context->engine(), _q_result);
     }
     break;
@@ -336,7 +315,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
 
     case 14:
     if (context->argumentCount() == 0) {
-        const QXmlStreamAttribute& _q_result = _q_self->last();
+        QXmlStreamAttribute _q_result = _q_self->last();
         return qScriptValueFromValue(context->engine(), _q_result);
     }
     break;
@@ -356,27 +335,20 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     break;
 
     case 16:
-    if (context->argumentCount() == 0) {
-        int _q_result = _q_self->length();
-        return QScriptValue(context->engine(), _q_result);
-    }
-    break;
-
-    case 17:
     if (context->argumentCount() == 1) {
         int _q_arg0 = context->argument(0).toInt32();
-        QVector<QXmlStreamAttribute > _q_result = _q_self->mid(_q_arg0);
+        QVector<QXmlStreamAttribute> _q_result = _q_self->mid(_q_arg0);
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     if (context->argumentCount() == 2) {
         int _q_arg0 = context->argument(0).toInt32();
         int _q_arg1 = context->argument(1).toInt32();
-        QVector<QXmlStreamAttribute > _q_result = _q_self->mid(_q_arg0, _q_arg1);
+        QVector<QXmlStreamAttribute> _q_result = _q_self->mid(_q_arg0, _q_arg1);
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     break;
 
-    case 18:
+    case 17:
     if (context->argumentCount() == 1) {
         QVector<QXmlStreamAttribute> _q_arg0;
         qScriptValueToSequence(context->argument(0), _q_arg0);
@@ -385,7 +357,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 19:
+    case 18:
     if (context->argumentCount() == 1) {
         QXmlStreamAttribute _q_arg0 = qscriptvalue_cast<QXmlStreamAttribute>(context->argument(0));
         _q_self->prepend(_q_arg0);
@@ -393,7 +365,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 20:
+    case 19:
     if (context->argumentCount() == 1) {
         int _q_arg0 = context->argument(0).toInt32();
         _q_self->remove(_q_arg0);
@@ -407,29 +379,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 21:
-    if (context->argumentCount() == 1) {
-        int _q_arg0 = context->argument(0).toInt32();
-        _q_self->removeAt(_q_arg0);
-        return context->engine()->undefinedValue();
-    }
-    break;
-
-    case 22:
-    if (context->argumentCount() == 0) {
-        _q_self->removeFirst();
-        return context->engine()->undefinedValue();
-    }
-    break;
-
-    case 23:
-    if (context->argumentCount() == 0) {
-        _q_self->removeLast();
-        return context->engine()->undefinedValue();
-    }
-    break;
-
-    case 24:
+    case 20:
     if (context->argumentCount() == 2) {
         int _q_arg0 = context->argument(0).toInt32();
         QXmlStreamAttribute _q_arg1 = qscriptvalue_cast<QXmlStreamAttribute>(context->argument(1));
@@ -438,7 +388,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 25:
+    case 21:
     if (context->argumentCount() == 1) {
         int _q_arg0 = context->argument(0).toInt32();
         _q_self->reserve(_q_arg0);
@@ -446,7 +396,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 26:
+    case 22:
     if (context->argumentCount() == 1) {
         int _q_arg0 = context->argument(0).toInt32();
         _q_self->resize(_q_arg0);
@@ -454,7 +404,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 27:
+    case 23:
     if (context->argumentCount() == 1) {
         bool _q_arg0 = context->argument(0).toBoolean();
         _q_self->setSharable(_q_arg0);
@@ -462,21 +412,21 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 28:
+    case 24:
     if (context->argumentCount() == 0) {
         int _q_result = _q_self->size();
         return QScriptValue(context->engine(), _q_result);
     }
     break;
 
-    case 29:
+    case 25:
     if (context->argumentCount() == 0) {
         _q_self->squeeze();
         return context->engine()->undefinedValue();
     }
     break;
 
-    case 30:
+    case 26:
     if (context->argumentCount() == 1) {
         QXmlStreamAttribute _q_arg0 = qscriptvalue_cast<QXmlStreamAttribute>(context->argument(0));
         bool _q_result = _q_self->startsWith(_q_arg0);
@@ -484,45 +434,14 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 31:
-    if (context->argumentCount() == 1) {
-        QVector<QXmlStreamAttribute> _q_arg0;
-        qScriptValueToSequence(context->argument(0), _q_arg0);
-        _q_self->swap(_q_arg0);
-        return context->engine()->undefinedValue();
-    }
-    break;
-
-    case 32:
-    if (context->argumentCount() == 1) {
-        int _q_arg0 = context->argument(0).toInt32();
-        QXmlStreamAttribute _q_result = _q_self->takeAt(_q_arg0);
-        return qScriptValueFromValue(context->engine(), _q_result);
-    }
-    break;
-
-    case 33:
+    case 27:
     if (context->argumentCount() == 0) {
-        QXmlStreamAttribute _q_result = _q_self->takeFirst();
-        return qScriptValueFromValue(context->engine(), _q_result);
-    }
-    break;
-
-    case 34:
-    if (context->argumentCount() == 0) {
-        QXmlStreamAttribute _q_result = _q_self->takeLast();
-        return qScriptValueFromValue(context->engine(), _q_result);
-    }
-    break;
-
-    case 35:
-    if (context->argumentCount() == 0) {
-        QList<QXmlStreamAttribute > _q_result = _q_self->toList();
+        QList<QXmlStreamAttribute> _q_result = _q_self->toList();
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     break;
 
-    case 36:
+    case 28:
     if (context->argumentCount() == 1) {
         QString _q_arg0 = context->argument(0).toString();
         QStringRef _q_result = _q_self->value(_q_arg0);
@@ -544,7 +463,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_prototype_call(QScriptContext
     }
     break;
 
-    case 37: {
+    case 29: {
     QString result = QString::fromLatin1("QXmlStreamAttributes");
     return QScriptValue(context->engine(), result);
     }
@@ -568,8 +487,9 @@ static QScriptValue qtscript_QXmlStreamAttributes_static_call(QScriptContext *co
         return context->throwError(QString::fromLatin1("QXmlStreamAttributes(): Did you forget to construct with 'new'?"));
     }
     if (context->argumentCount() == 0) {
-        QXmlStreamAttributes _q_cpp_result;
-        QScriptValue _q_result = context->engine()->newVariant(context->thisObject(), qVariantFromValue(_q_cpp_result));
+        QtScriptShell_QXmlStreamAttributes _q_cpp_result;
+        QScriptValue _q_result = context->engine()->newVariant(context->thisObject(), qVariantFromValue((QXmlStreamAttributes)_q_cpp_result));
+        _q_cpp_result.__qtscript_self = _q_result;
         return _q_result;
     }
     break;
@@ -578,7 +498,7 @@ static QScriptValue qtscript_QXmlStreamAttributes_static_call(QScriptContext *co
     if (context->argumentCount() == 1) {
         QList<QXmlStreamAttribute> _q_arg0;
         qScriptValueToSequence(context->argument(0), _q_arg0);
-        QVector<QXmlStreamAttribute > _q_result = QXmlStreamAttributes::fromList(_q_arg0);
+        QVector<QXmlStreamAttribute> _q_result = QXmlStreamAttributes::fromList(_q_arg0);
         return qScriptValueFromSequence(context->engine(), _q_result);
     }
     break;
@@ -595,7 +515,7 @@ QScriptValue qtscript_create_QXmlStreamAttributes_class(QScriptEngine *engine)
 {
     engine->setDefaultPrototype(qMetaTypeId<QXmlStreamAttributes*>(), QScriptValue());
     QScriptValue proto = engine->newVariant(qVariantFromValue((QXmlStreamAttributes*)0));
-    for (int i = 0; i < 38; ++i) {
+    for (int i = 0; i < 30; ++i) {
         QScriptValue fun = engine->newFunction(qtscript_QXmlStreamAttributes_prototype_call, qtscript_QXmlStreamAttributes_function_lengths[i+2]);
         fun.setData(QScriptValue(engine, uint(0xBABE0000 + i)));
         proto.setProperty(QString::fromLatin1(qtscript_QXmlStreamAttributes_function_names[i+2]),
