@@ -7,36 +7,48 @@
 #include <qabstractitemmodel.h>
 #include <qbytearray.h>
 #include <qcoreevent.h>
-#include <qdatastream.h>
 #include <qlist.h>
 #include <qmimedata.h>
 #include <qobject.h>
 #include <qsize.h>
 #include <qstringlist.h>
+#include <qvector.h>
 
 #define QTSCRIPT_IS_GENERATED_FUNCTION(fun) ((fun.data().toUInt32() & 0xFFFF0000) == 0xBABE0000)
 
-Q_DECLARE_METATYPE(QModelIndex)
+Q_DECLARE_METATYPE(const QMimeData*)
+Q_DECLARE_METATYPE(Qt::DropAction)
 Q_DECLARE_METATYPE(QChildEvent*)
 Q_DECLARE_METATYPE(QEvent*)
-Q_DECLARE_METATYPE(QMimeData*)
-Q_DECLARE_METATYPE(Qt::DropAction)
 Q_DECLARE_METATYPE(QFlags<Qt::ItemFlag>)
 Q_DECLARE_METATYPE(Qt::Orientation)
 template <> \
-struct QMetaTypeId< QMap<int,QVariant> > \
+struct QMetaTypeId< QMap<int,QVariant > > \
 { \
     enum { Defined = 1 }; \
     static int qt_metatype_id() \
     { \
         static QBasicAtomicInt metatype_id = Q_BASIC_ATOMIC_INITIALIZER(0); \
-        if (!metatype_id) \
-            metatype_id = qRegisterMetaType< QMap<int,QVariant> >("QMap<int,QVariant>"); \
-        return metatype_id; \
+        if (!metatype_id.load()) \
+            metatype_id.store(qRegisterMetaType< QMap<int,QVariant > >("QMap<int,QVariant >")); \
+        return metatype_id.load(); \
     } \
 };
 Q_DECLARE_METATYPE(QFlags<Qt::MatchFlag>)
-Q_DECLARE_METATYPE(QList<QModelIndex>)
+//Q_DECLARE_METATYPE(QList<QModelIndex >)
+Q_DECLARE_METATYPE(QMimeData*)
+template <> \
+struct QMetaTypeId< QHash<int,QByteArray > > \
+{ \
+    enum { Defined = 1 }; \
+    static int qt_metatype_id() \
+    { \
+        static QBasicAtomicInt metatype_id = Q_BASIC_ATOMIC_INITIALIZER(0); \
+        if (!metatype_id.load()) \
+            metatype_id.store(qRegisterMetaType< QHash<int,QByteArray > >("QHash<int,QByteArray >")); \
+        return metatype_id.load(); \
+    } \
+};
 Q_DECLARE_METATYPE(Qt::SortOrder)
 Q_DECLARE_METATYPE(QFlags<Qt::DropAction>)
 Q_DECLARE_METATYPE(QTimerEvent*)
@@ -60,6 +72,24 @@ QModelIndex  QtScriptShell_QAbstractListModel::buddy(const QModelIndex&  index)
     }
 }
 
+bool  QtScriptShell_QAbstractListModel::canDropMimeData(const QMimeData*  data, Qt::DropAction  action, int  row, int  column, const QModelIndex&  parent) const
+{
+    QScriptValue _q_function = __qtscript_self.property("canDropMimeData");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("canDropMimeData") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::canDropMimeData(data, action, row, column, parent);
+    } else {
+        QScriptEngine *_q_engine = __qtscript_self.engine();
+        return qscriptvalue_cast<bool >(_q_function.call(__qtscript_self,
+            QScriptValueList()
+            << qScriptValueFromValue(_q_engine, const_cast<const QMimeData *>(data))
+            << qScriptValueFromValue(_q_engine, action)
+            << qScriptValueFromValue(_q_engine, row)
+            << qScriptValueFromValue(_q_engine, column)
+            << qScriptValueFromValue(_q_engine, parent)));
+    }
+}
+
 bool  QtScriptShell_QAbstractListModel::canFetchMore(const QModelIndex&  parent) const
 {
     QScriptValue _q_function = __qtscript_self.property("canFetchMore");
@@ -127,7 +157,7 @@ bool  QtScriptShell_QAbstractListModel::dropMimeData(const QMimeData*  data, Qt:
         QScriptEngine *_q_engine = __qtscript_self.engine();
         return qscriptvalue_cast<bool >(_q_function.call(__qtscript_self,
             QScriptValueList()
-            << qScriptValueFromValue(_q_engine, const_cast<QMimeData *>(data))
+            << qScriptValueFromValue(_q_engine, const_cast<const QMimeData *>(data))
             << qScriptValueFromValue(_q_engine, action)
             << qScriptValueFromValue(_q_engine, row)
             << qScriptValueFromValue(_q_engine, column)
@@ -313,6 +343,42 @@ QStringList  QtScriptShell_QAbstractListModel::mimeTypes() const
     }
 }
 
+bool  QtScriptShell_QAbstractListModel::moveColumns(const QModelIndex&  sourceParent, int  sourceColumn, int  count, const QModelIndex&  destinationParent, int  destinationChild)
+{
+    QScriptValue _q_function = __qtscript_self.property("moveColumns");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("moveColumns") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::moveColumns(sourceParent, sourceColumn, count, destinationParent, destinationChild);
+    } else {
+        QScriptEngine *_q_engine = __qtscript_self.engine();
+        return qscriptvalue_cast<bool >(_q_function.call(__qtscript_self,
+            QScriptValueList()
+            << qScriptValueFromValue(_q_engine, sourceParent)
+            << qScriptValueFromValue(_q_engine, sourceColumn)
+            << qScriptValueFromValue(_q_engine, count)
+            << qScriptValueFromValue(_q_engine, destinationParent)
+            << qScriptValueFromValue(_q_engine, destinationChild)));
+    }
+}
+
+bool  QtScriptShell_QAbstractListModel::moveRows(const QModelIndex&  sourceParent, int  sourceRow, int  count, const QModelIndex&  destinationParent, int  destinationChild)
+{
+    QScriptValue _q_function = __qtscript_self.property("moveRows");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("moveRows") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::moveRows(sourceParent, sourceRow, count, destinationParent, destinationChild);
+    } else {
+        QScriptEngine *_q_engine = __qtscript_self.engine();
+        return qscriptvalue_cast<bool >(_q_function.call(__qtscript_self,
+            QScriptValueList()
+            << qScriptValueFromValue(_q_engine, sourceParent)
+            << qScriptValueFromValue(_q_engine, sourceRow)
+            << qScriptValueFromValue(_q_engine, count)
+            << qScriptValueFromValue(_q_engine, destinationParent)
+            << qScriptValueFromValue(_q_engine, destinationChild)));
+    }
+}
+
 bool  QtScriptShell_QAbstractListModel::removeColumns(int  column, int  count, const QModelIndex&  parent)
 {
     QScriptValue _q_function = __qtscript_self.property("removeColumns");
@@ -356,6 +422,17 @@ void QtScriptShell_QAbstractListModel::revert()
     }
 }
 
+QHash<int , QByteArray >  QtScriptShell_QAbstractListModel::roleNames() const
+{
+    QScriptValue _q_function = __qtscript_self.property("roleNames");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("roleNames") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::roleNames();
+    } else {
+        return qscriptvalue_cast<QHash<int , QByteArray > >(_q_function.call(__qtscript_self));
+    }
+}
+
 int  QtScriptShell_QAbstractListModel::rowCount(const QModelIndex&  parent) const
 {
     QScriptValue _q_function = __qtscript_self.property("rowCount");
@@ -418,6 +495,22 @@ bool  QtScriptShell_QAbstractListModel::setItemData(const QModelIndex&  index, c
     }
 }
 
+QModelIndex  QtScriptShell_QAbstractListModel::sibling(int  row, int  column, const QModelIndex&  idx) const
+{
+    QScriptValue _q_function = __qtscript_self.property("sibling");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("sibling") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::sibling(row, column, idx);
+    } else {
+        QScriptEngine *_q_engine = __qtscript_self.engine();
+        return qscriptvalue_cast<QModelIndex >(_q_function.call(__qtscript_self,
+            QScriptValueList()
+            << qScriptValueFromValue(_q_engine, row)
+            << qScriptValueFromValue(_q_engine, column)
+            << qScriptValueFromValue(_q_engine, idx)));
+    }
+}
+
 void QtScriptShell_QAbstractListModel::sort(int  column, Qt::SortOrder  order)
 {
     QScriptValue _q_function = __qtscript_self.property("sort");
@@ -458,6 +551,17 @@ bool  QtScriptShell_QAbstractListModel::submit()
     }
 }
 
+Qt::DropActions  QtScriptShell_QAbstractListModel::supportedDragActions() const
+{
+    QScriptValue _q_function = __qtscript_self.property("supportedDragActions");
+    if (!_q_function.isFunction() || QTSCRIPT_IS_GENERATED_FUNCTION(_q_function)
+        || (__qtscript_self.propertyFlags("supportedDragActions") & QScriptValue::QObjectMember)) {
+        return QAbstractListModel::supportedDragActions();
+    } else {
+        return qscriptvalue_cast<Qt::DropActions >(_q_function.call(__qtscript_self));
+    }
+}
+
 Qt::DropActions  QtScriptShell_QAbstractListModel::supportedDropActions() const
 {
     QScriptValue _q_function = __qtscript_self.property("supportedDropActions");
