@@ -4,12 +4,13 @@
 #include <qvalidator.h>
 
 #include <QtScript/qscriptvalue.h>
+#include <__package_shared.h>
 
 class QtScriptShell_QRegExpValidator : public QRegExpValidator
 {
 public:
     QtScriptShell_QRegExpValidator(QObject*  parent = 0);
-    QtScriptShell_QRegExpValidator(const QRegExp&  rx, QObject*  parent);
+    QtScriptShell_QRegExpValidator(const QRegExp&  rx, QObject*  parent = 0);
     ~QtScriptShell_QRegExpValidator();
 
     void childEvent(QChildEvent*  arg__1);
