@@ -23,25 +23,23 @@
 #include "core_global.h"
 
 #include <QList>
-#include <QStack>
 
 #include "RBox.h"
 #include "RColor.h"
-#include "RDebug.h"
 #include "REntityData.h"
-#include "RLayer.h"
 #include "RLineweight.h"
-#include "RMatrix.h"
+#include "RLinetypePattern.h"
 #include "RObject.h"
-#include "RPropertyAttributes.h"
+#include "RPolyline.h"
 #include "RPropertyTypeId.h"
+#include "RRefPoint.h"
 #include "RVector.h"
-#include "RExplodable.h"
 
+class RBlock;
 class RDocument;
-class REntity;
-class RBlockReferenceEntity;
 class RExporter;
+class RLayer;
+class RPropertyAttributes;
 class RViewportData;
 
 #ifndef RDEFAULT_QSET_INT
@@ -112,7 +110,13 @@ public:
         getData().setDocument(document);
     }
 
-    virtual REntity* clone() const = 0;
+    virtual QSharedPointer<RObject> clone() const = 0;
+
+    QSharedPointer<REntity> cloneToEntity() const {
+        QSharedPointer<REntity> ret = clone().dynamicCast<REntity>();
+        Q_ASSERT(!ret.isNull());
+        return ret;
+    }
 
     virtual RS::EntityType getType() const {
         return getData().getType();
@@ -148,7 +152,7 @@ public:
      * as part of the block definition (currently only block attributes).
      * The default implementation returns false.
      */
-//    virtual bool isPartOfBlockReference(REntity::Id blockRefId) const {
+//    virtual bool isPartOfBlockReference(RObject::Id blockRefId) const {
 //        Q_UNUSED(blockRefId)
 
 //        return false;
@@ -212,7 +216,7 @@ public:
     /**
      * \copydoc REntityData::setLayerId
      */
-    void setLayerId(RLayer::Id layerId) {
+    void setLayerId(RObject::Id layerId) {
         getData().setLayerId(layerId);
     }
 
@@ -226,7 +230,7 @@ public:
     /**
      * \copydoc REntityData::getLayerId
      */
-    RLayer::Id getLayerId() const {
+    RObject::Id getLayerId() const {
         return getData().getLayerId();
     }
 
@@ -240,14 +244,14 @@ public:
     /**
      * \copydoc REntityData::setBlockId
      */
-    void setBlockId(RBlock::Id blockId) {
+    void setBlockId(RObject::Id blockId) {
         getData().setBlockId(blockId);
     }
 
     /**
      * \copydoc REntityData::getBlockId
      */
-    RBlock::Id getBlockId() const {
+    RObject::Id getBlockId() const {
         return getData().getBlockId();
     }
 
@@ -261,14 +265,14 @@ public:
     /**
      * \copydoc REntityData::getParentId
      */
-    REntity::Id getParentId() const {
+    RObject::Id getParentId() const {
         return getData().getParentId();
     }
 
     /**
      * \copydoc REntityData::setLinetypeId
      */
-    void setLinetypeId(RLinetype::Id linetypeId) {
+    void setLinetypeId(RObject::Id linetypeId) {
         getData().setLinetypeId(linetypeId);
     }
 
@@ -282,12 +286,11 @@ public:
     /**
      * \copydoc REntityData::getLinetypeId
      */
-    RLinetype::Id getLinetypeId() const {
+    RObject::Id getLinetypeId() const {
         return getData().getLinetypeId();
     }
 
-    RLinetype::Id getLinetypeId(bool resolve,
-        const QStack<REntity*>& blockRefStack) const;
+    RObject::Id getLinetypeId(bool resolve, const QStack<QSharedPointer<REntity> >& blockRefStack) const;
 
     /**
      * \copydoc REntityData::getLinetypePattern
@@ -324,10 +327,9 @@ public:
         return getData().getLineweight();
     }
 
-    RLineweight::Lineweight getLineweight(bool resolve,
-        const QStack<REntity*>& blockRefStack) const;
+    RLineweight::Lineweight getLineweight(bool resolve, const QStack<QSharedPointer<REntity> >& blockRefStack) const;
 
-    double getLineweightInUnits(const QStack<REntity*>& blockRefStack) const;
+    double getLineweightInUnits(const QStack<QSharedPointer<REntity> >& blockRefStack) const;
 
     /**
      * \copydoc REntityData::setColor
@@ -343,8 +345,8 @@ public:
         return getData().getColor();
     }
 
-    RColor getColor(const RColor& unresolvedColor, const QStack<REntity*>& blockRefStack);
-    RColor getColor(bool resolve, const QStack<REntity*>& blockRefStack);
+    RColor getColor(const RColor& unresolvedColor, const QStack<QSharedPointer<REntity> >& blockRefStack);
+    RColor getColor(bool resolve, const QStack<QSharedPointer<REntity> >& blockRefStack);
 
     RColor getDisplayColor() {
         return getData().getDisplayColor();
@@ -391,7 +393,7 @@ public:
     /**
      * \copydoc REntityData::getIdBoundingBoxes
      */
-//    virtual QList<QPair<REntity::Id, RBox> > getIdBoundingBoxes() const {
+//    virtual QList<QPair<RObject::Id, RBox> > getIdBoundingBoxes() const {
 //        return getData().getIdBoundingBoxes();
 //    }
 
@@ -461,7 +463,7 @@ public:
     /**
      * \copydoc REntityData::getInternalReferencePoints
      */
-    virtual QList<RRefPoint> getInternalReferencePoints(RS::ProjectionRenderingHint hint=RS::RenderTop, QList<REntity::Id>* subEntityIds = NULL) const {
+    virtual QList<RRefPoint> getInternalReferencePoints(RS::ProjectionRenderingHint hint=RS::RenderTop, QList<RObject::Id>* subEntityIds = NULL) const {
         return getData().getInternalReferencePoints(hint, subEntityIds);
     }
 
@@ -496,7 +498,7 @@ public:
     /**
      * \copydoc REntityData::getCenterPoints
      */
-    virtual QList<RVector> getCenterPoints(const RBox& queryBox = RDEFAULT_RBOX, QList<REntity::Id>* subEntityIds = NULL) const {
+    virtual QList<RVector> getCenterPoints(const RBox& queryBox = RDEFAULT_RBOX, QList<RObject::Id>* subEntityIds = NULL) const {
         return getData().getCenterPoints(queryBox, subEntityIds);
     }
 
@@ -504,7 +506,7 @@ public:
      * \copydoc REntityData::getClosestPointOnEntity
      */
     virtual RVector getClosestPointOnEntity(const RVector& point,
-        double range=RNANDOUBLE, bool limited=true, REntity::Id* subEntityId = NULL) const {
+        double range=RNANDOUBLE, bool limited=true, RObject::Id* subEntityId = NULL) const {
 
         return getData().getClosestPointOnEntity(point, range, limited, subEntityId);
     }
@@ -522,7 +524,7 @@ public:
     /**
      * \copydoc REntityData::getIntersectionPoints(const REntity&, bool)
      */
-    virtual QList<RVector> getIntersectionPoints(const REntity& other, bool limited = true, const RBox& queryBox = RDEFAULT_RBOX, bool ignoreComplex = true, QList<QPair<REntity::Id, REntity::Id> >* entityIds = NULL) const;
+    virtual QList<RVector> getIntersectionPoints(const REntity& other, bool limited = true, const RBox& queryBox = RDEFAULT_RBOX, bool ignoreComplex = true, QList<QPair<RObject::Id, RObject::Id> >* entityIds = NULL) const;
 
     /**
      * \copydoc REntityData::getIntersectionPoints(const RShape&, bool)
@@ -630,7 +632,7 @@ public:
     /**
      * \copydoc REntityData::update
      */
-    void update() const {
+    virtual void update() const {
         getData().update();
     }
 
@@ -666,7 +668,7 @@ public:
     virtual bool setProperty(RPropertyTypeId propertyTypeId,
             const QVariant& value, RTransaction* transaction=NULL);
 
-    virtual bool isVisible(RBlock::Id blockId = RBlock::INVALID_ID) const;
+    virtual bool isVisible(RObject::Id blockId = RObject::INVALID_ID) const;
     virtual bool isEditable(bool allowInvisible = false) const;
     virtual bool isInWorkingSet() const;
 
@@ -679,7 +681,7 @@ protected:
      * \copydoc REntityData::setParentId
      * Use RStorage::setEntityParentId instead.
      */
-    void setParentId(REntity::Id parentId) {
+    void setParentId(RObject::Id parentId) {
         getData().setParentId(parentId);
     }
 
@@ -689,7 +691,7 @@ protected:
 Q_DECLARE_METATYPE(REntity*)
 Q_DECLARE_METATYPE(QSharedPointer<REntity>)
 Q_DECLARE_METATYPE(QSharedPointer<REntity>*)
-Q_DECLARE_METATYPE(QStack<REntity*>)
-Q_DECLARE_METATYPE(QStack<REntity*>*)
+Q_DECLARE_METATYPE(QStack<QSharedPointer<REntity> >)
+Q_DECLARE_METATYPE(QStack<QSharedPointer<REntity> >*)
 
 #endif
