@@ -19,16 +19,17 @@
 #include <QDir>
 #include <QFileInfo>
 #include <QList>
+#include <QMap>
 #include <QMenu>
 #include <QPainter>
 #include <QSvgRenderer>
 #include <QToolBar>
-//#include <QVectorIterator>
 
-#include "RDebug.h"
 #include "RGuiAction.h"
+#include "RDocument.h"
+#include "RS.h"
 #include "RScriptHandler.h"
-#include "RSingleApplication.h"
+#include "RSettings.h"
 #include "RWidget.h"
 
 
@@ -244,11 +245,26 @@ void RGuiAction::updateIcons() {
     }
 }
 
+void RGuiAction::updateToolTips() {
+    QList<RGuiAction*> actions = getActions();
+    for (int i=0; i<actions.length(); i++) {
+        RGuiAction* action = actions[i];
+        if (action==NULL) {
+            continue;
+        }
+        action->initTexts();
+    }
+}
+
 void RGuiAction::updateIcon() {
     setIcon(iconFile);
 }
 
 void RGuiAction::setIcon(const QString& iconFile) {
+    QString previousIcon = this->iconFile;
+
+    // save the original icon file
+    // used to update icons on theme or dark mode change:
     this->iconFile = iconFile;
 
     // look up theme specific icon:
@@ -271,11 +287,13 @@ void RGuiAction::setIcon(const QString& iconFile) {
     }
 
     if (themeIconFile.isEmpty()) {
+        // no icon set and no theme icon found:
         QAction::setIcon(QIcon());
     }
     else {
         QString fileName = themeIconFile;
 
+        // change fileName to dark mode icon if available (-inverse postfix):
         if (RSettings::hasDarkGuiBackground()) {
             QFileInfo fi(themeIconFile);
             QString iconFileDark = fi.absolutePath() + QDir::separator() + fi.baseName() + "-inverse." + fi.suffix();
@@ -284,31 +302,42 @@ void RGuiAction::setIcon(const QString& iconFile) {
             }
         }
 
-        if (QFileInfo(fileName).suffix().toLower()=="svg" &&
-            !QCoreApplication::arguments().contains("-max-icon-res")) {
+        // fileName can be the an original icon file or an icon file of the theme
+        // either with or without -inverse postfix:
+        if (QFileInfo(fileName).exists()) {
+            // scale icon down to maximum required resolution:
+            if (QFileInfo(fileName).suffix().toLower()=="svg" &&
+                !QCoreApplication::arguments().contains("-max-icon-res")) {
 
-            int iconSize = RSettings::getIntValue("CadToolBar/IconSize", 32);
-            iconSize = qMax(iconSize, RSettings::getIntValue("ToolBar/IconSize", 32));
-            iconSize = qMax(iconSize, RSettings::getIntValue("CadToolMatrix/IconSize", 24));
+                int iconSize = RSettings::getIntValue("CadToolBar/IconSize", 32);
+                iconSize = qMax(iconSize, RSettings::getIntValue("ToolBar/IconSize", 32));
+                iconSize = qMax(iconSize, RSettings::getIntValue("CadToolMatrix/IconSize", 24));
 
-            // retina icons:
-            if (RSettings::getDevicePixelRatio()>1) {
-                iconSize*=RSettings::getDevicePixelRatio();
+                // retina icons:
+                if (RSettings::getDevicePixelRatio()>1) {
+                    iconSize = (int)(RSettings::getDevicePixelRatio()*iconSize);
+                }
+
+                QPixmap pm(iconSize,iconSize);
+                pm.fill(Qt::transparent);
+                QPainter p;
+                p.begin(&pm);
+                QSvgRenderer renderer(fileName);
+                renderer.render(&p, QRectF(0, 0, iconSize, iconSize));
+                p.end();
+
+                QIcon icon(pm);
+                QAction::setIcon(icon);
+            }
+            else {
+                QAction::setIcon(QIcon(fileName));
             }
-
-            QPixmap pm(iconSize,iconSize);
-            pm.fill(Qt::transparent);
-            QPainter p;
-            p.begin(&pm);
-            QSvgRenderer renderer(fileName);
-            renderer.render(&p, QRectF(0, 0, iconSize, iconSize));
-            p.end();
-
-            QIcon icon(pm);
-            QAction::setIcon(icon);
         }
         else {
-            QAction::setIcon(QIcon(themeIconFile));
+            // icon not found:
+            // this is not an error, icons are not required:
+            // revert to previous icon:
+            this->iconFile = previousIcon;
         }
     }
 }
@@ -921,11 +950,22 @@ void RGuiAction::setScriptFile(const QString& sf, bool isSecondary) {
         relSf = dir.relativeFilePath(sf);
     }
     scriptFile = relSf;
-    setObjectName(QFileInfo(sf).completeBaseName() + "Action");
+    QFileInfo fi(sf);
+    setObjectName(fi.completeBaseName() + "Action");
 
     if (!isSecondary) {
         actionsByScriptFile[relSf] = this;
     }
+
+    // implicitely set icon file if available:
+    QString iconFilePath = fi.path() + QDir::separator() + fi.completeBaseName() + ".svg";
+    //if (QFileInfo(iconFilePath).exists()) {
+        setIcon(iconFilePath);
+    //}
+}
+
+QString RGuiAction::getScriptClass() const {
+    return QFileInfo(scriptFile).baseName();
 }
 
 void RGuiAction::setFactory(FactoryFunction f) {
@@ -1065,7 +1105,7 @@ RGuiAction* RGuiAction::getByScriptFile(const QString& scriptFile) {
         return actionsByScriptFile[pluginFilePath];
     }
 
-    qWarning() << "action not found:" << relFilePath;
+    //qWarning() << "action not found:" << relFilePath;
     return NULL;
 }
 
@@ -1234,7 +1274,7 @@ bool RGuiAction::slotTrigger(const QString& command) {
 
 
 
-/** 
+/**
  * Cleans up all GUI action objects.
  */
 void RGuiAction::clear() {
