@@ -16,8 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
-
-#include <QtGlobal>
+#include <QLineEdit>
 
 #if QT_VERSION >= 0x050000
 #include <QtConcurrent>
@@ -25,14 +24,15 @@
 #include <qtconcurrentmap.h>
 #endif
 
-#include "RColor.h"
-#include "RDebug.h"
-#include "RGlobal.h"
+#include "RBlockReferenceEntity.h"
+#include "RDocument.h"
+#include "RDocumentInterface.h"
 #include "RMath.h"
 #include "RMainWindow.h"
-#include "RMetaTypes.h"
+#include "RProperty.h"
+#include "RS.h"
+#include "RSettings.h"
 #include "RPropertyEditor.h"
-#include "RPropertyEvent.h"
 
 RPropertyEditor* RPropertyEditor::instance = NULL;
 
@@ -42,9 +42,14 @@ RPropertyEditor* RPropertyEditor::instance = NULL;
 RPropertyEditor::RPropertyEditor()
     : guiUpToDate(false),
       updatesDisabled(false),
-      entityTypeFilter(RS::EntityAll) {
+      entityTypeFilter(RS::EntityAll),
+      terminate(false) {
 
     instance = this;
+
+#if QT_VERSION >= 0x060000
+    connect(this, &RPropertyEditor::finished, this, &RPropertyEditor::updateFromDocumentNow);
+#endif
 }
 
 /**
@@ -228,6 +233,28 @@ void RPropertyEditor::updateProperty(const RPropertyTypeId& propertyTypeId,
 //    }
 //}
 
+#if QT_VERSION >= 0x060000
+void RPropertyEditor::sleep(QPromise<void>& promise, RDocument* document, bool onlyChanges, RS::EntityType filter, bool manual, bool showOnRequest) {
+    terminate = false;
+
+    QElapsedTimer t;
+    t.start();
+    int delay = RSettings::getIntValue("PropertyEditor/LazyUpdateDelay", 200);
+    while (t.elapsed() < delay) {
+        QThread::msleep(10);
+        if (terminate) {
+            break;
+        }
+    }
+
+    if (terminate) {
+        return;
+    }
+
+    emit finished(document, onlyChanges, filter, manual, showOnRequest);
+}
+#endif
+
 /**
  * Implements updateFromDocument from RPropertyListener to show properties for selected objects.
  *
@@ -247,8 +274,48 @@ void RPropertyEditor::updateFromDocument(RDocument* document, bool onlyChanges,
         return;
     }
 
-    //qDebug() << "RPropertyEditor::updateFromDocument";
-    //RDebug::startTimer();
+    showOnRequest = showOnRequest || RSettings::getPropertyEditorShowOnRequest()==true;
+
+#if QT_VERSION >= 0x060000
+    bool gotSelection = document->hasSelection();
+    int lazyUpdate = RSettings::getBoolValue("PropertyEditor/LazyUpdate", true);
+    if (lazyUpdate && !showOnRequest && gotSelection) {
+        // sleep, then update property editor if sleep was not interrupted by other call:
+        static QFuture<void> future = QFuture<void>();
+
+        if (future.isRunning()) {
+            // selection has changed before the property editor started to update:
+            // cancel and restart timer:
+            terminate = true;
+            future.cancel();
+            future.waitForFinished();
+
+            // don't clear
+            // call might be followed by update of values only:
+            //clearEditor();
+        }
+
+        future = QtConcurrent::run(&RPropertyEditor::sleep, this, document, onlyChanges, filter, manual, showOnRequest);
+    }
+    else {
+#endif
+
+        updateFromDocumentNow(document, onlyChanges, filter, manual, showOnRequest);
+
+#if QT_VERSION >= 0x060000
+    }
+#endif
+}
+
+void RPropertyEditor::updateFromDocumentNow(RDocument* document, bool onlyChanges, RS::EntityType filter, bool manual, bool showOnRequest) {
+    if (updatesDisabled) {
+        return;
+    }
+
+    if (document == NULL) {
+        clearEditor();
+        return;
+    }
 
     if (filter!=RS::EntityUnknown) {
         setEntityTypeFilter(filter);
@@ -489,9 +556,9 @@ void RPropertyEditor::updateFromDocument(RDocument* document, bool onlyChanges,
     QtConcurrent::blockingMap(ccProperties, RPropertyEditor::computePropertyValue);
 
     // serial:
-//    for (int i=0; i<ccProperties.length(); i++) {
-//        RPropertyEditor::computePropertyValue(ccProperties[i]);
-//    }
+    // for (int i=0; i<ccProperties.length(); i++) {
+    //     RPropertyEditor::computePropertyValue(ccProperties[i]);
+    // }
 
     // update combined properties map:
     for (int i=0; i<ccProperties.length(); i++) {
@@ -700,12 +767,12 @@ void RPropertyEditor::updateLayers(RDocumentInterface* documentInterface) {
     if (documentInterface==NULL) {
         // 20190130: changed from false to true:
         // workaround for Qt 5.10.1 bug (crash when changing layer name in property editor)
-        updateFromDocument(NULL, true);
+        updateFromDocumentNow(NULL, true);
     }
     else {
         // 20190130: changed from false to true:
         // workaround for Qt 5.10.1 bug (crash when changing layer name in property editor)
-        updateFromDocument(&documentInterface->getDocument(), true);
+        updateFromDocumentNow(&documentInterface->getDocument(), true);
     }
 }
 
