@@ -18,17 +18,16 @@
  */
 #include <QFont>
 #include <QFontMetricsF>
-//#include <QTextBlock>
-#include <QTextDocument>
 #include <QMutex>
 
 #include "RColor.h"
 #include "RDxfServices.h"
 #include "RFont.h"
 #include "RFontList.h"
+#include "RLine.h"
+#include "RTextBasedData.h"
 #include "RTextRenderer.h"
 #include "RPainterPathDevice.h"
-#include "RPolyline.h"
 #include "RSettings.h"
 
 QChar RTextRenderer::chDegree = QChar(0x00b0);
@@ -95,6 +94,8 @@ QString RTextRenderer::rxTrackChangeStr = "\\\\T(\\d*\\.?\\d*);";
 QRegularExpression RTextRenderer::rxTrackChange(rxTrackChangeStr);
 QString RTextRenderer::rxAlignmentChangeStr = "\\\\A(\\d+);";
 QRegularExpression RTextRenderer::rxAlignmentChange(rxAlignmentChangeStr);
+QString RTextRenderer::rxAlignmentChangeBlockStr = "\\\\A(\\d+)";
+QRegularExpression RTextRenderer::rxAlignmentChangeBlock(rxAlignmentChangeStr);
 QString RTextRenderer::rxFontChangeCadStr = "\\\\F([^|;]*)"    // \Ffont
                                          "[^;]*"            // ignore anything except ";"
                                          ";";               // require ";"
@@ -176,6 +177,7 @@ QString rxAllTmp =
     + RTextRenderer::rxObliqueAngleChangeStr + "|"
     + RTextRenderer::rxTrackChangeStr + "|"
     + RTextRenderer::rxAlignmentChangeStr + "|"
+    + RTextRenderer::rxAlignmentChangeBlockStr + "|"
     + RTextRenderer::rxFontChangeCadStr + "|"
     + RTextRenderer::rxFontChangeTtfStr + "|"
     + RTextRenderer::rxBeginBlockStr + "|"
@@ -965,6 +967,11 @@ void RTextRenderer::render() {
                             continue;
                         }
 
+                        // replace special characters in sub/super script:
+                        script.replace(rxDiameter, chDiameter);
+                        script.replace(rxPlusMinus, chPlusMinus);
+                        script.replace(rxDegree, chDegree);
+
                         QList<RPainterPath> paths;
 
                         double ascent = 0.0;
@@ -2125,7 +2132,7 @@ QList<RPainterPath> RTextRenderer::getPainterPathsForBlockCad(
                 RPainterPath path;
                 path.moveTo(line.getStartPoint());
                 path.lineTo(line.getEndPoint());
-                path.addOriginalShape(QSharedPointer<RShape>(line.clone()));
+                path.addOriginalShape(line.clone());
                 preparePathTransform(path, 0.0, 1.0);
                 ppBlock.addPath(path);
                 line = RLine();
