@@ -16,15 +16,21 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
+#include <typeinfo>
+
+#include "RBlock.h"
+#include "RBlockReferenceEntity.h"
 #include "RDocument.h"
-#include "RExporter.h"
+#include "RLayer.h"
 #include "RLinetype.h"
 #include "RLinkedStorage.h"
 #include "RMainWindow.h"
-#include "RMemoryStorage.h"
+#include "RS.h"
 #include "RStorage.h"
 #include "RSpline.h"
 #include "RTransaction.h"
+#include "RView.h"
+#include "RViewportEntity.h"
 
 RTransaction::RTransaction()
     : types(Generic),
@@ -41,6 +47,7 @@ RTransaction::RTransaction()
       existingBlockDetectionDisabled(false),
       existingLayerDetectionDisabled(false),
       blockRecursionDetectionDisabled(false),
+      deletingBlock(false),
       keepHandles(false),
       keepChildren(false),
       undoing(false),
@@ -66,6 +73,7 @@ RTransaction::RTransaction(RStorage& storage)
       existingBlockDetectionDisabled(false),
       existingLayerDetectionDisabled(false),
       blockRecursionDetectionDisabled(false),
+      deletingBlock(false),
       keepHandles(false),
       keepChildren(false),
       undoing(false),
@@ -102,6 +110,7 @@ RTransaction::RTransaction(
       existingBlockDetectionDisabled(false),
       existingLayerDetectionDisabled(false),
       blockRecursionDetectionDisabled(false),
+      deletingBlock(false),
       keepHandles(false),
       keepChildren(false),
       undoing(false),
@@ -139,6 +148,7 @@ RTransaction::RTransaction(
       existingBlockDetectionDisabled(false),
       existingLayerDetectionDisabled(false),
       blockRecursionDetectionDisabled(false),
+      deletingBlock(false),
       keepHandles(false),
       keepChildren(false),
       undoing(false),
@@ -225,6 +235,8 @@ void RTransaction::redo() {
 
             storage->removeObject(storage->queryObjectDirect(objId));
 
+            object->setAutoUpdatesBlocked(true);
+
             QList<RPropertyChange> objectChanges = propertyChanges.value(objId);
             for (int i=0; i<objectChanges.size(); ++i) {
                 RPropertyTypeId propertyTypeId = objectChanges.at(i).propertyTypeId;
@@ -237,7 +249,7 @@ void RTransaction::redo() {
                 QSharedPointer<REntity> entity = object.dynamicCast<REntity>();
                 if (!spatialIndexDisabled && !entity.isNull()) {
                     // entity of the block might have changed (in block drag and drop):
-                    if (entity->getType()==RS::EntityBlockRef) {
+                    if (entity->isOfType(RS::EntityBlockRef)) {
                         //entity->update();
                         affectedBlockReferenceIds.insert(objId);
                     }
@@ -245,6 +257,8 @@ void RTransaction::redo() {
                     document->addToSpatialIndex(entity);
                 }
             }
+
+            object->setAutoUpdatesBlocked(false);
         }
     }
 
@@ -328,7 +342,7 @@ void RTransaction::undo() {
                 QSharedPointer<REntity> entity = object.dynamicCast<REntity>();
                 if (!spatialIndexDisabled && !entity.isNull()) {
                     // entity of the block might have changed (in block drag and drop):
-                    if (entity->getType()==RS::EntityBlockRef) {
+                    if (entity->isOfType(RS::EntityBlockRef)) {
                         //entity->update();
                         affectedBlockReferenceIds.insert(objId);
                     }
@@ -616,7 +630,7 @@ bool RTransaction::addObject(QSharedPointer<RObject> object,
     // through transactions:
     // TODO: check if entity is editable:
     if (mustClone) {
-        QSharedPointer<RObject> clone = QSharedPointer<RObject>(object->clone());
+        QSharedPointer<RObject> clone = object->clone();
 
         objectStorage->setObjectId(*clone, RObject::INVALID_ID);
         // note that we delete the OLD entity here
@@ -686,7 +700,8 @@ bool RTransaction::addObject(QSharedPointer<RObject> object,
     bool objectIsBlock = false;
     if (object->getType()==RS::ObjectBlock) {
         QSharedPointer<RBlock> block = object.dynamicCast<RBlock>();
-        if (!block.isNull()) {
+        // blocks that are owned by references are copied with the reference:
+        if (!block.isNull() && !block->isOwnedByReference()) {
             objectIsBlock = true;
 
             if (!existingBlockDetectionDisabled) {
@@ -698,6 +713,61 @@ bool RTransaction::addObject(QSharedPointer<RObject> object,
         }
     }
 
+    // if object is a new block reference that owns its block, add a new block for this block reference to own:
+    if (object->getId()==RObject::INVALID_ID) {
+        if (object->isOfType(RS::EntityBlockRef)) {
+            QSharedPointer<RBlockReferenceEntity> blockRef = object.dynamicCast<RBlockReferenceEntity>();
+            if (!blockRef.isNull()) {
+                QString referencedBlockName = blockRef->getReferencedBlockName();
+
+                // find out if the block we would clone was added in this very transaction:
+                bool blockAdded =
+                    affectedObjectIds.contains(blockRef->getReferencedBlockId()) && statusChanges.contains(blockRef->getReferencedBlockId());
+
+                // if (!storageIsLinked) {
+                //     qDebug() << "blockRef->hasBlockOwnership():" << blockRef->hasBlockOwnership();
+                //     qDebug() << "affectedObjectIds.contains(blockRef->getReferencedBlockId()):" << affectedObjectIds.contains(blockRef->getReferencedBlockId());
+                //     qDebug() << "blockRef->getReferencedBlockId():" << blockRef->getReferencedBlockId();
+                //     qDebug() << "statusChanges.contains(blockRef->getReferencedBlockId()):" << statusChanges.contains(blockRef->getReferencedBlockId());
+                //     qDebug() << "doc->hasBlock(referencedBlockName):" << doc->hasBlock(referencedBlockName);
+                // }
+
+                // only clone block if block reference owns block and block exists already and was not added in this transaction:
+                // block might not exist when copying from one document to another (e.g. clipboard):
+                // don't clone blocks during imports (keepHandles is true):
+                if (blockRef->hasBlockOwnership() && doc->hasBlock(referencedBlockName) && !blockAdded && !keepHandles) {
+                    QSharedPointer<RBlock> block = doc->queryBlock(blockRef->getReferencedBlockId());
+                    if (block.isNull()) {
+                        qWarning() << "RTransaction::addObject: "
+                            "block reference owns block that does not exist";
+                        fail();
+                        return false;
+                    }
+
+                    // clone block to avoid changing the original block:
+                    QSharedPointer<RBlock> newBlock = block->cloneToBlock();
+                    QString newBlockName = doc->getUniqueBlockName(block->getName(), usedBlockNames);
+                    if (!storageIsLinked) qDebug() << "20241213: clone block " << referencedBlockName << " to " << newBlockName;
+                    //qDebug() << "newBlockName: " << newBlockName;
+                    newBlock->setName(newBlockName);
+                    usedBlockNames.append(newBlockName);
+                    addObject(newBlock, false, true);
+
+                    // copy block contents to new block:
+                    QSet<REntity::Id> entityIds = doc->queryBlockEntities(block->getId());
+                    for (QSet<REntity::Id>::iterator it=entityIds.begin(); it!=entityIds.end(); ++it) {
+                        QSharedPointer<REntity> entity = doc->queryEntityDirect(*it);
+                        QSharedPointer<REntity> entityClone = entity->cloneToEntity();
+                        entityClone->setBlockId(newBlock->getId());
+                        addObject(entityClone, false, true);
+                    }
+
+                    blockRef->setReferencedBlockId(newBlock->getId());
+                }
+            }
+        }
+    }
+
     // if object is a layer,
     // look up existing layer based on case insensitive name comparison:
     if (object->getType()==RS::ObjectLayer) {
@@ -866,7 +936,9 @@ bool RTransaction::addObject(QSharedPointer<RObject> object,
             onlyChanges = false;
         }
 
-        ret = storage->saveObject(object, !blockRecursionDetectionDisabled, keepHandles);
+
+        bool overwrite = false;
+        ret = storage->saveObject(object, !blockRecursionDetectionDisabled, keepHandles, &overwrite);
 
         if (!ret) {
             qCritical() << "RTransaction::addObject: saveObject() failed for object: ";
@@ -893,12 +965,13 @@ bool RTransaction::addObject(QSharedPointer<RObject> object,
                     // update block references if entity has been changed inside a block
                     // that is not the current block (in block editing):
                     if (entity->getBlockId()!=storage->getCurrentBlockId()) {
+                        // add block of entity to blocks that need a spatial index update:
                         affectedBlockReferenceIds.unite(storage->queryBlockReferences(entity->getBlockId()));
                     }
                 }
             }
 
-            if (!objectHasChanged) {
+            if (!objectHasChanged && !overwrite) {
                 statusChanges.insert(object->getId());
             }
         }
@@ -1024,7 +1097,7 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
         return;
     }
 
-    // if a layer is deleted, delete all entities on the layer:
+    // if a layer is deleted, delete all entities on that layer:
     QSharedPointer<RLayer> layer = object.dynamicCast<RLayer>();
     if (!layer.isNull()) {
         if (layer->getName() == "0") {
@@ -1042,10 +1115,37 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
 
         // TODO: undeletable layers
 
-        QSet<REntity::Id> ids = storage->queryLayerEntities(objectId, true);
-        QSetIterator<REntity::Id> it(ids);
-        while (it.hasNext()) {
-            deleteObject(it.next(), true);
+        {
+            QSet<REntity::Id> ids = storage->queryLayerEntities(objectId, true);
+            QSetIterator<REntity::Id> it(ids);
+            while (it.hasNext()) {
+                deleteObject(it.next(), true);
+            }
+        }
+
+        // delete all viewport references to that layer:
+        // 202411
+        {
+            QSet<REntity::Id> ids = storage->queryAllEntities(false, true, RS::EntityViewport);
+            QSetIterator<REntity::Id> it(ids);
+            while (it.hasNext()) {
+                // remove layer from list of frozen layers:
+                REntity::Id viewportId = it.next();
+                QSharedPointer<REntity> e = storage->queryEntity(viewportId);
+                if (!e.isNull()) {
+                    QSharedPointer<RViewportEntity> viewport = e.dynamicCast<RViewportEntity>();
+                    if (!viewport.isNull()) {
+                        QList<RLayer::Id> frozenLayerIds = viewport->getFrozenLayerIds();
+
+                        if (!frozenLayerIds.isEmpty()) {
+                            frozenLayerIds.removeAll(layer->getId());
+                            viewport->setFrozenLayerIds(frozenLayerIds);
+
+                            addObject(viewport, false, false);
+                        }
+                    }
+                }
+            }
         }
 
         // current layer deleted, reset current layer to layer "0":
@@ -1054,7 +1154,7 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
         }
     }
 
-    // if a block is deleted, delete all entities in the block:
+    // if a block is deleted, delete all block references and entities in the block:
     QSharedPointer<RBlock> block = object.dynamicCast<RBlock> ();
     if (!block.isNull()) {
         if (block->getName() == RBlock::modelSpaceName) {
@@ -1063,6 +1163,8 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
             return;
         }
 
+        deletingBlock = true;
+
         // make sure entities / block references on locked layers are deleted:
         bool allowAllOri = allowAll;
         allowAll = true;
@@ -1087,10 +1189,11 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
         }
 
         allowAll = allowAllOri;
+
+        deletingBlock = false;
     }
 
     QSharedPointer<REntity> entity = object.dynamicCast<REntity>();
-
     if (!entity.isNull()) {
         if (!allowAll && !force && !entity->isEditable(allowInvisible)) {
             qWarning() << "RTransaction::deleteObject: entity not editable (locked or hidden layer)";
@@ -1114,6 +1217,20 @@ void RTransaction::deleteObject(QSharedPointer<RObject> object, bool force) {
         allowAll = allowAllOri;
     }
 
+    // prevent recursion while deleting block references with block ownership:
+    if (!entity.isNull() && !deletingBlock) {
+        QSharedPointer<RBlockReferenceEntity> blockRef = object.dynamicCast<RBlockReferenceEntity>();
+        if (!blockRef.isNull()) {
+            // if block reference with ownership of block is deleted, delete block:
+            if (blockRef->hasBlockOwnership()) {
+                // delete block:
+                // deleting the block will also delete all references to it:
+                deleteObject(blockRef->getReferencedBlockId(), true);
+                return;
+            }
+        }
+    }
+
     // if the current view is deleted, reset current view:
     QSharedPointer<RView> view = object.dynamicCast<RView>();
     if (!view.isNull()) {
@@ -1149,6 +1266,12 @@ bool RTransaction::isPreview() const {
     return (ls!=NULL);
 }
 
+RObject::Id RTransaction::getNewObjectId(RObject::Id oldId) const {
+    if (cloneIds.contains(oldId)) {
+        return cloneIds[oldId];
+    }
+    return RObject::INVALID_ID;
+}
 
 /**
  * \return List of property changes by this transaction for the given object.
