@@ -18,9 +18,12 @@
  */
 #include <cmath>
 
+#include <QDebug>
+
 #include "RArc.h"
 #include "RCircle.h"
 #include "RBox.h"
+#include "RLine.h"
 #include "RMath.h"
 #include "RPolyline.h"
 
@@ -202,7 +205,7 @@ QList<RArc> RArc::createBiarc(const RVector& startPoint, double startDirection,
     double beta = RMath::getAngleDifference180(angle, endDirection);
 
     double theta;
-    if ((alpha>=0 && beta>=0) || (alpha<=0 && beta<=0)) {
+    if ((alpha>0 && beta>0) || (alpha<0 && beta<0)) {
         // same sign: C-shaped curve:
         theta = alpha;
     }
@@ -301,27 +304,45 @@ RS::Side RArc::getSideOfPoint(const RVector& point) const {
     }
 }
 
-void RArc::moveStartPoint(const RVector& pos) {
-    double bulge = getBulge();
-
-    // full circle: trim instead of move:
-    if (bulge < 1.0e-6 || bulge > 1.0e6) {
-        startAngle = center.getAngleTo(pos);
+void RArc::moveStartPoint(const RVector& pos, bool keepRadius) {
+    if (!keepRadius) {
+        RArc a = RArc::createFrom3Points(pos, getMiddlePoint(), getEndPoint());
+        if (a.isReversed()!=isReversed()) {
+            a.reverse();
+        }
+        *this = a;
     }
     else {
-        *this = RArc::createFrom2PBulge(pos, getEndPoint(), bulge);
+        double bulge = getBulge();
+
+        // full circle: trim instead of move:
+        if (bulge < 1.0e-6 || bulge > 1.0e6) {
+            startAngle = center.getAngleTo(pos);
+        }
+        else {
+            *this = RArc::createFrom2PBulge(pos, getEndPoint(), bulge);
+        }
     }
 }
 
-void RArc::moveEndPoint(const RVector& pos) {
-    double bulge = getBulge();
-
-    // full circle: trim instead of move:
-    if (bulge < 1.0e-6 || bulge > 1.0e6) {
-        endAngle = center.getAngleTo(pos);
+void RArc::moveEndPoint(const RVector& pos, bool keepRadius) {
+    if (!keepRadius) {
+        RArc a = RArc::createFrom3Points(pos, getMiddlePoint(), getStartPoint());
+        if (a.isReversed()!=isReversed()) {
+            a.reverse();
+        }
+        *this = a;
     }
     else {
-        *this = RArc::createFrom2PBulge(getStartPoint(), pos, bulge);
+        double bulge = getBulge();
+
+        // full circle: trim instead of move:
+        if (bulge < 1.0e-6 || bulge > 1.0e6) {
+            endAngle = center.getAngleTo(pos);
+        }
+        else {
+            *this = RArc::createFrom2PBulge(getStartPoint(), pos, bulge);
+        }
     }
 }
 
@@ -530,6 +551,10 @@ void RArc::setReversed(bool r) {
 }
 
 RBox RArc::getBoundingBox() const {
+    if (!isValid()) {
+        return RBox();
+    }
+
     RVector minV;
     RVector maxV;
     double minX = qMin(getStartPoint().x, getEndPoint().x);
@@ -671,7 +696,7 @@ RVector RArc::getVectorTo(const RVector& point, bool limited, double strictRange
         return RVector::invalid;
     }
 
-    RVector v = point - center;
+    RVector v = (point - center).get2D();
     return RVector::createPolar(v.getMagnitude() - radius, v.getAngle());
 }
 
@@ -1002,22 +1027,22 @@ QList<QSharedPointer<RShape> > RArc::splitAt(const QList<RVector>& points) const
         sortedPoints.append(endPoint);
     }
 
-    //qDebug() << "sortedPoints: " << sortedPoints;
-
     for (int i=0; i<sortedPoints.length()-1; i++) {
         if (sortedPoints[i].equalsFuzzy(sortedPoints[i+1])) {
             continue;
         }
 
-        RArc* seg = clone();
-        double a1 = center.getAngleTo(sortedPoints[i]);
-        double a2 = center.getAngleTo(sortedPoints[i+1]);
-        if (fabs(RMath::getAngleDifference180(a1, a2)*radius)<0.001) {
-            continue;
+        QSharedPointer<RArc> seg = clone().dynamicCast<RArc>();
+        if (!seg.isNull()) {
+            double a1 = center.getAngleTo(sortedPoints[i]);
+            double a2 = center.getAngleTo(sortedPoints[i+1]);
+            if (fabs(RMath::getAngleDifference180(a1, a2)*radius)<0.001) {
+                continue;
+            }
+            seg->setStartAngle(a1);
+            seg->setEndAngle(a2);
+            ret.append(seg);
         }
-        seg->setStartAngle(a1);
-        seg->setEndAngle(a2);
-        ret.append(QSharedPointer<RShape>(seg));
     }
 
     return ret;
