@@ -16,7 +16,11 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
-#include <QtGlobal>
+#include <QDebug>
+#include <QCryptographicHash>
+#include <QLocale>
+#include <QFile>
+#include <QFileInfo>
 
 #ifdef Q_OS_WIN
 // for _isnan:
@@ -24,7 +28,7 @@
 #endif
 
 #include <complex>
-#include <QCryptographicHash>
+#include <cerrno>
 
 #if QT_VERSION >= 0x060000
 #  include <QJSEngine>
@@ -32,9 +36,10 @@
 #  include <QScriptEngine>
 #endif
 
-#include "RDebug.h"
 #include "RMath.h"
+#include "RS.h"
 #include "RSettings.h"
+#include "RVector.h"
 
 /**
  * Last error encountered.
