@@ -22,8 +22,8 @@
 
 #include "../core_global.h"
 
-#include <RS.h>
-#include <RMath.h>
+#include "RS.h"
+#include "RMath.h"
 
 class RArc;
 class RExporter;
@@ -73,6 +73,7 @@ public:
 
     virtual bool simplify(RPolyline& polyline, double tolerance) = 0;
     virtual QList<RPolyline> splitAtSegmentTypeChange(const RPolyline& polyline) = 0;
+    virtual bool insertVerticesAtSelfIntersections(RPolyline& polyline) = 0;
 };
 
 #endif
