@@ -18,10 +18,13 @@
  */
 #include <cmath>
 
+#include <QDebug>
+#include <QMutex>
+
 #include "RArc.h"
-#include "RDebug.h"
 #include "RLine.h"
 #include "RSpline.h"
+#include "RSplineProxy.h"
 #include "RPainterPath.h"
 #include "RPolyline.h"
 
@@ -958,6 +961,9 @@ void RSpline::appendToExploded(const RLine& line) const {
         return;
     }
 
+    static QMutex m;
+    QMutexLocker ml(&m);
+
     if (!exploded.isEmpty()) {
         // compare angle of this sement with last segment and
         // modify last segment if angle is the same (straight line):
@@ -1019,7 +1025,7 @@ double RSpline::getLength() const {
     }
 
     if (hasProxy()) {
-        length = splineProxy->getDistanceAtT(*this, getTMax());
+        length = splineProxy->getLength(*this);
     }
     else {
         length = 0.0;
@@ -1257,10 +1263,18 @@ bool RSpline::reverse() {
     int k;
     if (!isClosed()) {
         for (k = 0; k < controlPoints.size()/2; k++) {
+#if QT_VERSION >= 0x060000
+            controlPoints.swapItemsAt(k,controlPoints.size()-(1+k));
+#else
             controlPoints.swap(k,controlPoints.size()-(1+k));
+#endif
         }
         for (k = 0; k < fitPoints.size()/2; k++) {
+#if QT_VERSION >= 0x060000
+            fitPoints.swapItemsAt(k,fitPoints.size()-(1+k));
+#else
             fitPoints.swap(k,fitPoints.size()-(1+k));
+#endif
         }
         double t;
         int i, j;
@@ -1276,14 +1290,22 @@ bool RSpline::reverse() {
     else {
         if (hasFitPoints()) {
             for (k = 0; k < (int)floor(fitPoints.size()/2.0); k++) {
+#if QT_VERSION >= 0x060000
+                fitPoints.swapItemsAt(k,fitPoints.size()-(1+k));
+#else
                 fitPoints.swap(k,fitPoints.size()-(1+k));
+#endif
             }
             // keep start node the same:
             fitPoints.prepend(fitPoints.takeLast());
         }
         else {
             for (k = 0; k < controlPoints.size()/2; k++) {
+#if QT_VERSION >= 0x060000
+                controlPoints.swapItemsAt(k,controlPoints.size()-(1+k));
+#else
                 controlPoints.swap(k,controlPoints.size()-(1+k));
+#endif
             }
         }
         updateTangentsPeriodic();
@@ -1304,7 +1326,10 @@ bool RSpline::stretch(const RPolyline& area, const RVector& offset) {
 }
 
 QSharedPointer<RShape> RSpline::getTransformed(const QTransform& transform) const {
-    QSharedPointer<RSpline> ret = QSharedPointer<RSpline>(clone());
+    QSharedPointer<RSpline> ret = clone().dynamicCast<RSpline>();
+    if (ret.isNull()) {
+        return QSharedPointer<RShape>();
+    }
 
     for (int i=0; i<ret->controlPoints.size(); i++) {
         ret->controlPoints[i].transform2D(transform);
@@ -1882,14 +1907,14 @@ QList<QSharedPointer<RShape> > RSpline::splitAt(const QList<RVector>& points) co
 
     QList<QSharedPointer<RShape> > ret;
 
-    QMap<double, RVector> sortable;
+    QMultiMap<double, RVector> sortable;
     for (int i=0; i<points.length(); i++) {
         double t = getTAtPoint(points[i]);
         sortable.insert(t, points[i]);
     }
 
     QList<double> keys = sortable.keys();
-#if QT_VERSION > 0x050000
+#if QT_VERSION >= 0x060000
     std::sort(keys.begin(), keys.end());
 #else
     qSort(keys);
@@ -1905,13 +1930,24 @@ QList<QSharedPointer<RShape> > RSpline::splitAt(const QList<RVector>& points) co
 
     QList<RSpline> subSplines = splitAtPoints(sortedPoints);
     for (int i=0; i<subSplines.length(); i++) {
-        ret.append(QSharedPointer<RShape>(subSplines[i].clone()));
+        ret.append(subSplines[i].clone());
     }
     return ret;
 }
 
-QList<RVector> RSpline::getSelfIntersectionPoints() const {
-    return getIntersectionPointsSS(*this, *this, true, true);
+/**
+ * Finds _some_ self intersection points of splines. Used for snapping to those intersections.
+ * Note that this does not find all intersections and might also return non-intersections.
+ * Most notably, self-intersections of bezier segments are not detected.
+ * This is not suitable to reliably detect the existence of self intersections.
+ */
+QList<RVector> RSpline::getSelfIntersectionPoints(double tolerance) const {
+    return getIntersectionPointsSS(*this, *this, true, true, tolerance);
+}
+
+void RSpline::setSplineProxy(RSplineProxy* p) {
+    delete splineProxy;
+    splineProxy = p;
 }
 
 void RSpline::print(QDebug dbg) const {
