@@ -23,11 +23,16 @@
 #include "entity_global.h"
 
 #include "RColor.h"
-#include "RDocument.h"
+#include "RDimStyleData.h"
 #include "REntityData.h"
 #include "RTextData.h"
 #include "RVector.h"
 
+class RDocument;
+class RLine;
+class RRefPoint;
+class RBlockReferenceEntity;
+
 /**
  * Base class for dimension entity data classes.
  *
@@ -52,7 +57,7 @@ public:
                    double lineSpacingFactor,
                    const QString& text,
                    const QString& fontName,
-                   double textAngle);
+                   double textRotation);
 
 //    ~RDimensionData() {
 //        if (overrides!=NULL) {
@@ -542,8 +547,12 @@ public:
         return lineSpacingFactor;
     }
 
-    double getTextAngle() const {
-        return textAngle;
+    double getTextRotation() const {
+        return textRotation;
+    }
+
+    void setTextRotation(double r) {
+        textRotation = r;
     }
 
 //    QList<QSharedPointer<RShape> > getDimensionLineShapes(
@@ -605,7 +614,7 @@ protected:
 
     mutable double defaultAngle;
     /** Rotation angle of dimension text away from default orientation */
-    double textAngle;
+    double textRotation;
 
     // RDimStyleData for overrides:
     RDimStyleData overrides;
