@@ -16,27 +16,31 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
+#include <QDebug>
 #include <QPen>
+#include <QElapsedTimer>
 
 #include "RArc.h"
 #include "RCircle.h"
+#include "RDocument.h"
 #include "RLine.h"
 #include "REllipse.h"
 #include "RHatchData.h"
 #include "RPainterPath.h"
 #include "RPainterPathExporter.h"
-#include "RPointEntity.h"
 #include "RPatternListImperial.h"
 #include "RPatternListMetric.h"
+#include "RRefPoint.h"
 #include "RSettings.h"
 #include "RSpline.h"
-#include "RUnit.h"
 #include "RPolyline.h"
 
 RHatchProxy* RHatchData::hatchProxy = NULL;
 
 RHatchData::RHatchData() :
     solid(true),
+    winding(false),
+    autoRegen(true),
     scaleFactor(1.0),
     angle(0.0),
     patternName("SOLID"),
@@ -65,6 +69,8 @@ RHatchData::RHatchData(const RHatchData& other) : REntityData(other), RPainterPa
  */
 RHatchData::RHatchData(bool solid, double scaleFactor, double angle, const QString& patternName) :
     solid(solid),
+    winding(false),
+    autoRegen(true),
     scaleFactor(scaleFactor),
     angle(angle),
     patternName(patternName),
@@ -76,6 +82,8 @@ RHatchData& RHatchData::operator =(const RHatchData& other) {
     REntityData::operator=(other);
 
     solid = other.solid;
+    winding = other.winding;
+    autoRegen = other.autoRegen;
     scaleFactor = other.scaleFactor;
     angle = other.angle;
     transparency = other.transparency;
@@ -176,7 +184,7 @@ double RHatchData::getDistanceTo(const RVector& point, bool limited, double rang
 
     // path is filled (solid) or very dens / huge or we're in draft mode:
     int comp = getComplexity();
-    if (isSolid() || comp>10000 || painterPaths.isEmpty() || draft) {
+    if (isSolid() || comp>1000 || painterPaths.isEmpty() || draft) {
         if (boundaryPath.contains(QPointF(point.x, point.y))) {
             if (RMath::isNaN(ret) || strictRange<ret) {
                 ret = strictRange;
@@ -456,9 +464,10 @@ bool RHatchData::stretch(const RPolyline& area, const RVector& offset) {
     return ret;
 }
 
-QList<QSharedPointer<RShape> > RHatchData::getShapes(const RBox& queryBox, bool ignoreComplex, bool segment) const {
+QList<QSharedPointer<RShape> > RHatchData::getShapes(const RBox& queryBox, bool ignoreComplex, bool segment, QList<RObject::Id>* entityIds) const {
     Q_UNUSED(queryBox)
     Q_UNUSED(segment)
+    Q_UNUSED(entityIds)
 
     QList<QSharedPointer<RShape> > ret;
 
@@ -489,6 +498,76 @@ QList<QSharedPointer<RShape> > RHatchData::getExploded() const {
     return getShapes();
 }
 
+double RHatchData::getLength() const {
+    double ret = 0.0;
+//    QList<QSharedPointer<RShape> > shapes = getShapes();
+
+//    for (int i=0; i<shapes.length(); i++) {
+//        ret += shapes[i]->getLength();
+//    }
+
+//    return ret;
+
+    QList<RPolyline> pls = getBoundaryAsPolylines();
+
+    for (int i=0; i<pls.length(); i++) {
+        ret += pls[i].getLength();
+    }
+    return ret;
+}
+
+double RHatchData::getArea() const {
+    QList<RPolyline> pls = getBoundaryAsPolylines();
+
+    double ret = 0.0;
+    for (int i=0; i<pls.length(); i++) {
+        RPolyline pl = pls[i];
+
+        // check if polyline is inside odd number of other polylines (-)
+        // or even number of other polylines (+) of this hatch:
+        int counter = 0;
+        for (int k=0; k<pls.length(); k++) {
+            // don't check against polyline self:
+            if (i==k) {
+                continue;
+            }
+            RPolyline other = pls[k];
+
+            // define contains as 'must contain every vertex':
+            bool contains = true;
+            if (!other.getBoundingBox().contains(pl.getBoundingBox())) {
+                // bounding box of other does not contain polyline:
+                contains = false;
+            }
+            else {
+                RVector pInside = pl.getPointInside();
+                if (!pInside.isValid()) {
+                    pInside = pl.getStartPoint();
+                }
+                if (!other.contains(pInside, true, 1.0e-3)) {
+                    // point inside polyline is not inside other:
+                    contains = false;
+                }
+            }
+
+            if (contains) {
+                counter++;
+            }
+        }
+
+        if (counter%2==0) {
+            // island:
+            ret += pl.getArea();
+        }
+        else {
+            // hole:
+            ret -= pl.getArea();
+        }
+    }
+
+    return ret;
+}
+
 void RHatchData::clearCustomPattern() {
     pattern.clear();
     update();
@@ -607,9 +686,20 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
 
     painterPaths.clear();
 
+
     // for solids, return boundary path, which can be filled:
     // in draft mode, return boundary path to be shown without fill:
     if (isSolid() || draft) {
+        // prevent stalling at large zoom for complex solid fills (QPainterPath performance issue):
+        if (getComplexity()>RSettings::getMaxHatchComplexity()*100) {
+            pixelSizeHint*=100;
+        }
+        else if (getComplexity()>RSettings::getMaxHatchComplexity()) {
+            pixelSizeHint*=10;
+        }
+        if (getComplexity()>RSettings::getMaxHatchComplexity() && pixelSizeHint<0.001) {
+            pixelSizeHint = 0.001;
+        }
         getBoundaryPath(pixelSizeHint);
         if (draft) {
             boundaryPath.setPen(QPen(Qt::SolidLine));
@@ -620,6 +710,9 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
             boundaryPath.setBrush(QBrush(Qt::SolidPattern));
         }
         boundaryPath.setAutoRegen(true);
+        if (isWinding()) {
+            boundaryPath.setFillRule(Qt::WindingFill);
+        }
         painterPaths.append(boundaryPath);
         dirty = false;
         gotDraft = draft;
@@ -631,6 +724,15 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
         return painterPaths;
     }
 
+    int timeOut = RSettings::getMaxHatchTime();
+    if (timeOut==0) {
+        // time out of 0 disables hatch rendering:
+        return painterPaths;
+    }
+
+    //RDebug::startTimer();
+
+    //qDebug() << "regenerating hatch";
     //RDebug::startTimer();
 
     // get pattern:
@@ -677,13 +779,18 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
     bool hasDots = localPattern.hasDots();
 
     RBox boundaryBox = boundaryPath.getBoundingBox();
-    boundaryBox.grow(1.0);
+
+    // grow boundary by 1% of largest size:
+    double w = boundaryBox.getWidth();
+    double h = boundaryBox.getHeight();
+    double s = qMax(w, h);
+    boundaryBox.grow(s/100.0);
+
     QList<RLine> boundaryEdges = boundaryBox.getLines2d();
     QList<RVector> boundaryCorners = boundaryBox.getCorners2d();
 
-    QTime timer;
+    QElapsedTimer timer;
     timer.start();
-    int timeOut = -1;
 
     QList<RPatternLine> patternLines = localPattern.getPatternLines();
     for (int i=0; i<patternLines.length(); i++) {
@@ -833,12 +940,17 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
             // line split up into segments, cut at contour intersections:
             QList<RLine> segments = getSegments(unclippedLine);
 
+            // untrimmed for debugging:
+            //QList<RLine> segments;
+            //segments.append(unclippedLine);
+
             RPainterPathExporter ppExporter;
             //ppExporter.setExportZeroLinesAsPoints(false);
             ppExporter.setExportZeroLinesAsPoints(true);
             // ignore zero lines if
             // line was split up into segments
             ppExporter.setIgnoreZeroLines(!hasDots);
+            // ensure pattern scale of 1:
             ppExporter.setLineweight(RLineweight::Weight100);
             if (!patternLine.dashes.isEmpty()) {
                 RLinetypePattern pat;
@@ -867,10 +979,7 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
                 }
             }
 
-            if (timer.elapsed()>500) {
-                if (timeOut==-1) {
-                    timeOut = RSettings::getIntValue("GraphicsView/MaxHatchTime", 2000);
-                }
+            //if (timer.elapsed()>500) {
                 if (timer.elapsed()>timeOut) {
                     qWarning() << "RHatchData::getPainterPaths: hatch pattern too dense. hatch pattern generation aborted (timeout set to " << timeOut << ")...";
                     painterPaths.clear();
@@ -878,13 +987,16 @@ QList<RPainterPath> RHatchData::getPainterPaths(bool draft, double pixelSizeHint
                     gotDraft = draft;
                     return painterPaths;
                 }
-            }
+            //}
         }
     }
 
     dirty = false;
     gotDraft = draft;
 
+    //RDebug::stopTimer("regen hatch");
+    //RDebug::printBacktrace();
+
     return painterPaths;
 }
 
@@ -897,8 +1009,11 @@ RPainterPath RHatchData::getBoundaryPath(double pixelSizeHint) const {
     }
 
     boundaryPath = RPainterPath();
-    gotPixelSizeHint = pixelSizeHint;
-    boundaryPath.setPixelSizeHint(pixelSizeHint);
+
+    if (getAutoRegen()) {
+        gotPixelSizeHint = pixelSizeHint;
+        boundaryPath.setPixelSizeHint(pixelSizeHint);
+    }
 
     for (int i=0; i<boundary.size(); ++i) {
         QList<QSharedPointer<RShape> > loop = boundary.at(i);
@@ -949,7 +1064,9 @@ RPainterPath RHatchData::getBoundaryPath(double pixelSizeHint) const {
 
                 RPainterPathExporter exp;
                 exp.setExportZeroLinesAsPoints(false);
-                exp.setPixelSizeHint(pixelSizeHint*2);
+                if (getAutoRegen()) {
+                    exp.setPixelSizeHint(pixelSizeHint*2);
+                }
                 exp.exportArcSegment(*arc);
                 RPainterPath p = exp.getPainterPath();
                 boundaryPath.appendPath(p);
@@ -1255,12 +1372,18 @@ QList<RPolyline> RHatchData::getBoundaryAsPolylines(double segmentLength) const
 }
 
 int RHatchData::getComplexity() const {
-    QList<RPainterPath> pps = getPainterPaths(false);
+//    QList<RPainterPath> pps = getPainterPaths(false);
 
-    int ret = 0;
-    for (int i=0; i<pps.count(); i++) {
-        ret += pps[i].getElementCount();
+//    int ret = 0;
+//    for (int i=0; i<pps.count(); i++) {
+//        ret += pps[i].getElementCount();
+//    }
+
+//    return ret;
+
+    int total = 0;
+    for (int i=0; i<boundary.length(); i++) {
+        total += boundary[i].length();
     }
-
-    return ret;
+    return total;
 }
