@@ -1,6 +1,6 @@
 /**
  * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
- * 
+ *
  * This file is part of the QCAD project.
  *
  * QCAD is free software: you can redistribute it and/or modify
@@ -16,40 +16,33 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
-#include <QtCore>
 #include <QDragMoveEvent>
+#include <QFrame>
+#include <QGestureEvent>
 #include <QPainter>
-#include <QScrollBar>
-#include <QWheelEvent>
 
-#include "RDebug.h"
-#include "RDocument.h"
 #include "RDocumentInterface.h"
-#include "RFocusFrame.h"
-#include "RGuiAction.h"
-#include "RGraphicsScene.h"
-#include "RGraphicsSceneQt.h"
+#include "RGraphicsViewImage.h"
 #include "RGraphicsViewQt.h"
-#include "RLine.h"
+#include "RMainWindow.h"
 #include "RMainWindowQt.h"
-#include "RMdiChildQt.h"
 #include "RMouseEvent.h"
+#include "RS.h"
 #include "RSettings.h"
-#include "RSingleApplication.h"
-#include "RSnap.h"
+#include "RTabletEvent.h"
 #include "RTerminateEvent.h"
-#include "RUnit.h"
 #include "RWheelEvent.h"
 
+class RGraphicsScene;
 
 
 RGraphicsViewQt::RGraphicsViewQt(QWidget* parent, bool showFocus)
     : QWidget(parent),
-      RGraphicsViewImage(),
       showFocus(showFocus),
       focusFrameWidget(NULL),
       gesturesEnabled(true),
-      gotMouseButtonPress(false) {
+      gotMouseButtonPress(false),
+      imageView(NULL) {
 
     setFocusPolicy(Qt::WheelFocus);
     setMouseTracking(true);
@@ -65,27 +58,27 @@ RGraphicsViewQt::RGraphicsViewQt(QWidget* parent, bool showFocus)
     grabGesture(Qt::PanGesture);
     grabGesture(Qt::PinchGesture);
     setContextMenuPolicy(Qt::PreventContextMenu);
+
+    setImageView(new RGraphicsViewImage(this));
+
+    connect(imageView, &QObject::destroyed, this, &RGraphicsViewQt::imageViewDeleted);
 }
 
 
 
 RGraphicsViewQt::~RGraphicsViewQt() {
+//    if (imageView!=NULL) {
+//        delete imageView;
+//    }
 }
 
-/**
- * Triggers a paintEvent based on a buffered offscreen bitmap (very fast).
- */
-void RGraphicsViewQt::repaintView() {
-    update();
-}
-
-void RGraphicsViewQt::repaintNow() {
-    QWidget::repaint();
-}
-
-void RGraphicsViewQt::viewportChangeEvent() {
-    RGraphicsViewImage::viewportChangeEvent();
-    emit viewportChanged();
+void RGraphicsViewQt::setImageView(RGraphicsViewImage* v) {
+    if (imageView!=NULL) {
+        delete imageView;
+    }
+    imageView = v;
+    imageView->setParent(this);
+    imageView->setWidget(this);
 }
     
 /**
@@ -96,24 +89,35 @@ void RGraphicsViewQt::viewportChangeEvent() {
  * \see invalidate
  */
 void RGraphicsViewQt::paintEvent(QPaintEvent* e) {
-    RDocumentInterface* di = getDocumentInterface();
+    if (imageView==NULL) {
+        return;
+    }
+
+    double dpr = imageView->getDevicePixelRatio();
+
+    RDocumentInterface* di = imageView->getDocumentInterface();
     if (di!=NULL && di->isSuspended()) {
         QPainter wPainter(this);
-        wPainter.drawImage(getRect(), graphicsBufferWithPreview);
+        QImage img = imageView->getGraphicsBufferWithPreview();
+        if (!RMath::fuzzyCompare(dpr, 0.0)) {
+            img.setDevicePixelRatio(dpr);
+        }
+        wPainter.drawImage(0,0, img);
         wPainter.end();
         return;
     }
 
-    updateImage();
+    imageView->updateImage();
 
     // event is NULL for fake paint events (testing):
     if (e!=NULL) {
         QPainter wPainter(this);
-        //wPainter.drawImage(0, 0, graphicsBufferWithPreview);
-        wPainter.drawImage(getRect(), graphicsBufferWithPreview);
-        //QPixmap pm;
-        //pm.convertFromImage(graphicsBufferWithPreview);
-        //wPainter.drawPixmap(this->rect(), pm);
+        QImage img = imageView->getGraphicsBufferWithPreview();
+        if (!RMath::fuzzyCompare(dpr, 0.0)) {
+            img.setDevicePixelRatio(dpr);
+        }
+        wPainter.drawImage(0,0, img);
+
         wPainter.end();
     }
 }
@@ -124,7 +128,12 @@ void RGraphicsViewQt::paintEvent(QPaintEvent* e) {
 bool RGraphicsViewQt::event(QEvent* e) {
     RTerminateEvent* te = dynamic_cast<RTerminateEvent*>(e);
     if (te!=NULL) {
-        RGraphicsView::handleTerminateEvent(*te);
+        if (imageView!=NULL) {
+            imageView->handleTerminateEvent(*te);
+        }
+        else {
+            qWarning() << "imageView was deleted";
+        }
         return true;
     }
 
@@ -151,16 +160,20 @@ bool RGraphicsViewQt::event(QEvent* e) {
  * Handle gesture events.
  */
 bool RGraphicsViewQt::gestureEvent(QGestureEvent* event) {
+    if (imageView==NULL) {
+        return true;
+    }
+
     if (QGesture* swipe = event->gesture(Qt::SwipeGesture)) {
-        RGraphicsView::handleSwipeGestureEvent(*static_cast<QSwipeGesture *>(swipe));
+        imageView->handleSwipeGestureEvent(*static_cast<QSwipeGesture *>(swipe));
     }
 
     else if (QGesture* pan = event->gesture(Qt::PanGesture)) {
-        RGraphicsView::handlePanGestureEvent(*static_cast<QPanGesture *>(pan));
+        imageView->handlePanGestureEvent(*static_cast<QPanGesture *>(pan));
     }
 
     else if (QGesture* pinch = event->gesture(Qt::PinchGesture)) {
-        RGraphicsView::handlePinchGestureEvent(*static_cast<QPinchGesture *>(pinch));
+        imageView->handlePinchGestureEvent(*static_cast<QPinchGesture *>(pinch));
     }
 
     event->accept();
@@ -169,14 +182,20 @@ bool RGraphicsViewQt::gestureEvent(QGestureEvent* event) {
 }
 
 void RGraphicsViewQt::tabletEvent(QTabletEvent* event) {
-    if (event == NULL || scene==NULL) {
+    if (imageView==NULL) {
+        return;
+    }
+
+    RGraphicsScene* s = imageView->getScene();
+
+    if (event == NULL || s==NULL) {
         return;
     }
 
 #if QT_VERSION < 0x050C00
     // Qt 5.12, 5.13 crashes:
-    RTabletEvent e(*event, *scene, *this);
-    RGraphicsView::handleTabletEvent(e);
+    RTabletEvent e(*event, *s, *imageView);
+    imageView->handleTabletEvent(e);
 #endif
 
     // needed for some types of Wacom tablets under Windows:
@@ -187,12 +206,18 @@ void RGraphicsViewQt::tabletEvent(QTabletEvent* event) {
  * Relays the Qt mouse event to the scene.
  */
 void RGraphicsViewQt::mouseMoveEvent(QMouseEvent* event) {
-    if (event==NULL || scene==NULL) {
+    if (imageView==NULL) {
         return;
     }
 
-    RMouseEvent e(*event, *scene, *this, getDevicePixelRatio());
-    RGraphicsView::handleMouseMoveEvent(e);
+    if (event==NULL || imageView->getScene()==NULL) {
+        return;
+    }
+
+    RGraphicsScene* s = imageView->getScene();
+
+    RMouseEvent e(*event, *s, *imageView, imageView->getDevicePixelRatio());
+    imageView->handleMouseMoveEvent(e);
 
  #if (QT_VERSION < QT_VERSION_CHECK(4, 8, 0))
     // workaround for wacom tablets bug: wacom tablets don't fire a mouseReleaseEvent
@@ -244,8 +269,13 @@ void RGraphicsViewQt::mouseMoveEvent(QMouseEvent* event) {
 }
 
 void RGraphicsViewQt::mousePressEvent(QMouseEvent* event) {
+    if (imageView==NULL) {
+        return;
+    }
+
     gotMouseButtonPress = true;
-    if (event==NULL || scene==NULL) {
+    RGraphicsScene* s = imageView->getScene();
+    if (event==NULL || s==NULL) {
         return;
     }
 
@@ -257,11 +287,11 @@ void RGraphicsViewQt::mousePressEvent(QMouseEvent* event) {
 //    }
 //#endif
 
-    RMouseEvent e(*event, *scene, *this, getDevicePixelRatio());
+    RMouseEvent e(*event, *s, *imageView, imageView->getDevicePixelRatio());
     mousePressScreenPosition = e.getScreenPosition();
     mousePressModelPosition = e.getModelPosition();
     mouseClickTimer.start();
-    RGraphicsView::handleMousePressEvent(e);
+    imageView->handleMousePressEvent(e);
     lastButtonState = event->buttons();
     event->accept();
 }
@@ -270,6 +300,10 @@ void RGraphicsViewQt::mouseReleaseEvent(QMouseEvent* event) {
     static int ignoreTimeThreshold = RSettings::getIntValue("GraphicsView/IgnoreTimeThreshold", 150);
     static int ignoreDeltaThreshold = RSettings::getIntValue("GraphicsView/IgnoreDeltaThreshold", 100);
 
+    if (imageView==NULL) {
+        return;
+    }
+
     if (gotMouseButtonPress) {
         // everything OK, got press, followed by release
     }
@@ -282,7 +316,9 @@ void RGraphicsViewQt::mouseReleaseEvent(QMouseEvent* event) {
     }
     gotMouseButtonPress = false;
 
-    if (event==NULL || scene==NULL) {
+    RGraphicsScene* s = imageView->getScene();
+
+    if (event==NULL || s==NULL) {
         return;
     }
 
@@ -296,7 +332,8 @@ void RGraphicsViewQt::mouseReleaseEvent(QMouseEvent* event) {
 
     event->ignore();
 
-    RMouseEvent e(*event, *scene, *this, getDevicePixelRatio());
+
+    RMouseEvent e(*event, *s, *imageView, imageView->getDevicePixelRatio());
 
     // if mouse press and mouse release happened within a short time span and the
     // mouse movement was small, use the mouse press location
@@ -306,13 +343,19 @@ void RGraphicsViewQt::mouseReleaseEvent(QMouseEvent* event) {
         e.setScreenPosition(mousePressScreenPosition);
         e.setModelPosition(mousePressModelPosition);
     }
-    RGraphicsView::handleMouseReleaseEvent(e);
+    imageView->handleMouseReleaseEvent(e);
     lastButtonState = event->buttons();
     event->accept();
 }
 
 void RGraphicsViewQt::mouseDoubleClickEvent(QMouseEvent* event) {
-    if (event==NULL || scene==NULL) {
+    if (imageView==NULL) {
+        return;
+    }
+
+    RGraphicsScene* s = imageView->getScene();
+
+    if (event==NULL || s==NULL) {
         return;
     }
 
@@ -324,28 +367,47 @@ void RGraphicsViewQt::mouseDoubleClickEvent(QMouseEvent* event) {
 //    }
 //#endif
 
-    RMouseEvent e(*event, *scene, *this, getDevicePixelRatio());
-    RGraphicsView::handleMouseDoubleClickEvent(e);
+    RMouseEvent e(*event, *s, *imageView, imageView->getDevicePixelRatio());
+    imageView->handleMouseDoubleClickEvent(e);
     lastButtonState = event->buttons();
     event->accept();
 }
 
 void RGraphicsViewQt::wheelEvent(QWheelEvent* event) {
-    if (event==NULL || scene==NULL) {
+    if (imageView==NULL) {
         return;
     }
-    RWheelEvent e(*event, *scene, *this, getDevicePixelRatio());
-    RGraphicsView::handleWheelEvent(e);
+
+    RGraphicsScene* s = imageView->getScene();
+
+    if (event==NULL || s==NULL) {
+        return;
+    }
+    RWheelEvent e(*event, *s, *imageView, imageView->getDevicePixelRatio());
+    imageView->handleWheelEvent(e);
     event->accept();
 }
 
 void RGraphicsViewQt::keyPressEvent(QKeyEvent* event) {
+    if (imageView==NULL) {
+        return;
+    }
+
     if (event==NULL) {
         return;
     }
     event->ignore();
 
-    RGraphicsView::handleKeyPressEvent(*event);
+    imageView->handleKeyPressEvent(*event);
+
+    if (event->key()==Qt::Key_Tab) {
+        RMainWindowQt* appWin = RMainWindowQt::getMainWindow();
+        if (appWin!=NULL) {
+            if (appWin->handleTabKey(NULL)) {
+                event->accept();
+            }
+        }
+    }
 
     // we're NOT accepting the event here to make sure the
     // event handler of the main window has a chance to
@@ -353,12 +415,15 @@ void RGraphicsViewQt::keyPressEvent(QKeyEvent* event) {
 }
 
 void RGraphicsViewQt::keyReleaseEvent(QKeyEvent* event) {
+    if (imageView==NULL) {
+        return;
+    }
     if (event==NULL) {
         return;
     }
     event->ignore();
 
-    RGraphicsView::handleKeyReleaseEvent(*event);
+    imageView->handleKeyReleaseEvent(*event);
 
     // we're NOT accepting the event here to make sure the
     // event handler of the main window has a chance to
@@ -376,15 +441,20 @@ void RGraphicsViewQt::dropEvent(QDropEvent* event) {
 }
 
 void RGraphicsViewQt::resizeEvent(QResizeEvent* ) {
-    regenerate();
+    if (imageView==NULL || imageView->getDocument()==NULL) {
+        return;
+    }
+    double dpr = imageView->getDevicePixelRatio();
+    imageView->resizeImage((int)(width()*dpr), (int)(height()*dpr));
+    imageView->regenerate();
 }
 
 int RGraphicsViewQt::getWidth() const {
-    return width() * getDevicePixelRatio();
+    return width();
 }
 
 int RGraphicsViewQt::getHeight() const {
-    return height() * getDevicePixelRatio();
+    return height();
 }
 
 QRect RGraphicsViewQt::getRect() const {
@@ -399,18 +469,6 @@ void RGraphicsViewQt::setSignalsBlocked(bool on) {
     blockSignals(on);
 }
 
-QCursor RGraphicsViewQt::getCursor() {
-    return QWidget::cursor();
-}
-
-void RGraphicsViewQt::setCursor(Qt::CursorShape cursorShape) {
-    QWidget::setCursor(cursorShape);
-}
-
-void RGraphicsViewQt::setCursor(const QCursor& cursor) {
-    QWidget::setCursor(cursor);
-}
-
 QSize RGraphicsViewQt::sizeHint() const {
     return QSize(200, 200);
 }
@@ -420,16 +478,21 @@ QSize RGraphicsViewQt::sizeHint() const {
  * that events go to this view.
  */
 void RGraphicsViewQt::focusInEvent(QFocusEvent* event) {
-    if (getDocumentInterface()!=NULL) {
+    if (imageView==NULL) {
+        return;
+    }
+
+    RDocumentInterface* di = imageView->getDocumentInterface();
+    if (di!=NULL) {
         RGraphicsViewQt* other =
                 dynamic_cast<RGraphicsViewQt*>(
-                    getDocumentInterface()->getLastKnownViewWithFocus()
+                    di->getLastKnownViewWithFocus()
                 );
         if (other!=NULL) {
             other->removeFocus();
         }
 
-        getDocumentInterface()->setLastKnownViewWithFocus(this);
+        di->setLastKnownViewWithFocus(imageView);
 
         if (focusFrameWidget!=NULL) {
             QPalette p = focusFrameWidget->palette();
@@ -447,7 +510,7 @@ void RGraphicsViewQt::focusInEvent(QFocusEvent* event) {
 
         RMainWindow* mainWindow = RMainWindow::getMainWindow();
         if (mainWindow!=NULL) {
-            mainWindow->notifyViewFocusListeners(this);
+            mainWindow->notifyViewFocusListeners(imageView);
         }
     }
 
@@ -461,14 +524,6 @@ void RGraphicsViewQt::focusOutEvent(QFocusEvent* event) {
     QWidget::focusOutEvent(event);
 }
 
-void RGraphicsViewQt::giveFocus() {
-    QWidget::setFocus(Qt::OtherFocusReason);
-}
-
-bool RGraphicsViewQt::hasFocus() {
-    return QWidget::hasFocus();
-}
-
 void RGraphicsViewQt::removeFocus() {
     if (showFocus && focusFrameWidget!=NULL) {
         QPalette p = focusFrameWidget->parentWidget()->palette();
@@ -480,43 +535,22 @@ void RGraphicsViewQt::setFocusFrameWidget(QFrame* w) {
     focusFrameWidget = w;
 }
 
-void RGraphicsViewQt::emitUpdateSnapInfo(RSnap* snap, RSnapRestriction* restriction) {
-    if (receivers(SIGNAL(updateSnapInfo(QPainter*, RSnap*, RSnapRestriction*))) > 0) {
-        QPainter gbPainter(&graphicsBufferWithPreview);
-        emit(updateSnapInfo(&gbPainter, snap, restriction));
-        gbPainter.end();
-    }
-}
-
-void RGraphicsViewQt::emitUpdateTextLabel(const RTextLabel& textLabel) {
-    if (receivers(SIGNAL(updateTextLabel(QPainter*, const RTextLabel&))) > 0) {
-        QPainter gbPainter(&graphicsBufferWithPreview);
-        emit(updateTextLabel(&gbPainter, textLabel));
-        gbPainter.end();
-    }
-}
-
 void RGraphicsViewQt::simulateMouseMoveEvent() {
-    if (!lastKnownScreenPosition.isValid() && isVisible()) {
+    if (imageView==NULL) {
+        return;
+    }
+
+    if (!imageView->getLastKnownScreenPosition().isValid() && isVisible()) {
         QPoint p = mapFromGlobal(QCursor::pos());
         if (p.x()>width() || p.x()<0 || p.y()<0 || p.y()>height()) {
             p = QPoint(width()/2, height()/2);
         }
-        lastKnownScreenPosition = RVector(p.x(), p.y());
-        lastKnownModelPosition = mapFromView(lastKnownScreenPosition);
+        imageView->setLastKnownScreenPosition(RVector(p.x(), p.y()));
+        imageView->setLastKnownModelPosition(imageView->mapFromView(RVector(p.x(), p.y())));
     }
-    RGraphicsView::simulateMouseMoveEvent();
+    //imageView->simulateMouseMoveEvent();
 }
 
-double RGraphicsViewQt::getDevicePixelRatio() const {
-    if (RSettings::getHighResolutionGraphicsView()) {
-#if QT_VERSION >= 0x050000
-        return devicePixelRatio();
-#else
-        return 1;
-#endif
-    }
-    else {
-        return 1;
-    }
+void RGraphicsViewQt::imageViewDeleted() {
+    imageView = NULL;
 }
