@@ -16,26 +16,30 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
-#include <QtGui>
-#include <QDesktopWidget>
-#include <QLabel>
+#include <QComboBox>
+#include <QLineEdit>
 #include <QMenu>
 #include <QMdiArea>
+#include <QSettings>
+#include <QScreen>
 #include <QStatusBar>
 #include <QTabBar>
 #include <QToolBar>
 #include <QToolButton>
-#include <QIconDragEvent>
 
 #include <RSingleApplication.h>
 
-#include "RDebug.h"
+#include "RCloseCurrentEvent.h"
+#include "RCoordinateEvent.h"
+#include "RDocument.h"
 #include "RDocumentInterface.h"
 #include "RGuiAction.h"
+#include "RGraphicsViewImage.h"
 #include "RGraphicsViewQt.h"
 #include "RMainWindowQt.h"
 #include "RMdiArea.h"
 #include "RMdiChildQt.h"
+#include "RPropertyEvent.h"
 #include "RScriptHandler.h"
 #include "RSelectionChangedEvent.h"
 #include "RSettings.h"
@@ -275,13 +279,12 @@ void RMainWindowQt::updateScenes(QMdiSubWindow* mdiChild) {
 }
 
 void RMainWindowQt::closeEvent(QCloseEvent* e) {
-
-    // workaround for Qt 5.6.1, 5.6.2 bug:
+    // Part 2 of workaround for Qt 5.6.1, 5.6.2, 5.15.0, 6.x bug:
     // dock widget closes before close dialog is shown
     // dock widget state not persistent between sessions
     // dock widget closes if user cancels close dialog
 #ifdef Q_OS_MAC
-#if QT_VERSION >= 0x050601 && QT_VERSION <= 0x050602
+#if (QT_VERSION >= 0x050601 && QT_VERSION <= 0x050602) || (QT_VERSION >= 0x050F00 && QT_VERSION < 0x060000) || (QT_VERSION >= 0x060600 && QT_VERSION < 0x060801)
     // restore dock widgets that were already closed by the same event due to
     // a Qt bug:
     QString eventAddr = QString("0x%1").arg((qlonglong)e, 0, 16);
@@ -289,13 +292,21 @@ void RMainWindowQt::closeEvent(QCloseEvent* e) {
     for (int i=0; i<closedDocks.length(); i++) {
         QString closedDock = closedDocks[i];
         if (closedDock.startsWith(eventAddr + "|")) {
-            QString objName = closedDock.split("|").at(1);
+            QStringList tuples = closedDock.split("|");
+            if (tuples.length()!=3) {
+                qWarning() << "unexpected value in ClosedDocks:" << closedDocks;
+                continue;
+            }
+            QString objName = tuples.at(1);
+            int x = tuples.at(2).toInt();
             QWidget* w = findChild<QWidget*>(objName);
             if (w) {
                 w->setVisible(true);
+                w->move(x, w->y());
             }
         }
     }
+    setProperty("ClosedDocks", QStringList());
 #endif
 #endif
 
@@ -523,6 +534,10 @@ void RMainWindowQt::showContextMenu(RObject::Id entityId, const RVector& pos) {
     emit contextMenu((int)entityId, pos);
 }
 
+void RMainWindowQt::requestResourceBlockEditing(RObject::Id entityId, const RVector& pos) {
+    emit editResourceBlock((int)entityId, pos);
+}
+
 void RMainWindowQt::setGraphicsViewCursor(const QCursor& cursor) {
     if (mdiArea==NULL) {
         return;
@@ -578,23 +593,24 @@ bool RMainWindowQt::readSettings() {
         statusBar()->hide();
     }
 
-    // get total available width on all screens:
-    int totalWidth = 0;
-    for (int i=0; i<QApplication::desktop()->screenCount(); i++) {
-        totalWidth+=QApplication::desktop()->availableGeometry(i).width();
-    }
+    // sanity check:
+    const QList<QScreen*> screens = QGuiApplication::screens();
+    const QList<QWidget*> topLevelWidgets = QApplication::topLevelWidgets();
 
-    // sanity check for x:
-    if (x()>totalWidth-100) {
-        move(totalWidth-width(), y());
-    }
+    for (int i=0; i<topLevelWidgets.length(); i++) {
+        bool sane = false;
+        QWidget* widget = topLevelWidgets[i];
 
-    // make sure all tool bars are visible:
-    QList<QToolBar*> tbs = findChildren<QToolBar*>();
-    for (int i=0; i<tbs.length(); i++) {
-        QToolBar* tb = tbs[i];
-        if (tb->x()>totalWidth-50) {
-            tb->move(totalWidth - tb->width(), tb->y());
+        QRect widgetRect = widget->frameGeometry();
+        for (int k=0; k<screens.length(); k++) {
+            QScreen* screen = screens[k];
+            if (screen->geometry().intersects(widgetRect)) {
+                sane = true;
+            }
+        }
+
+        if (!sane) {
+            widget->move(0,0);
         }
     }
 
@@ -606,10 +622,13 @@ bool RMainWindowQt::readSettings() {
  */
 void RMainWindowQt::writeSettings() {
     RMainWindow::writeSettings();
+
     RSettings::getQSettings()->setValue("Appearance/DockappWindows", saveState());
     RSettings::getQSettings()->setValue("Appearance/FullScreen", isFullScreen());
     RSettings::getQSettings()->setValue("Appearance/Maximized", isMaximized());
     RSettings::getQSettings()->setValue("Appearance/StatusBar", statusBar()->isVisible());
+
+    RSettings::getQSettings()->sync();
 }
 
 QWidget* RMainWindowQt::getChildWidget(const QString& name) {
@@ -635,22 +654,57 @@ bool RMainWindowQt::event(QEvent* e) {
         return false;
     }
 
-    // TODO: never triggered on macOS when draging title bar icon
-//    QIconDragEvent* ide = dynamic_cast<QIconDragEvent*>(e);
-//    if (ide!=NULL) {
-//        qDebug() << "QIconDragEvent";
-//        ide->accept();
-//        return true;
-//    }
+    switch ((int)e->type()) {
 
-    if (e->type()==QEvent::PaletteChange) {
-        RSettings::resetCache();
-        RGuiAction::updateIcons();
-        notifyPaletteListeners();
-        update();
+    case QEvent::ScreenChangeInternal:
+    {
+        RDocumentInterface* di = getDocumentInterface();
+        if (di!=NULL) {
+            RGraphicsView* gv = di->getLastKnownViewWithFocus();
+            if (gv!=NULL) {
+                gv->regenerate();
+            }
+        }
     }
+    break;
 
-    if (e->type()==QEvent::KeyPress) {
+
+    case QEvent::WindowActivate:
+    {
+        // hand keyboard focus to grphics view for various key events (tab, entering number, options shortcuts):
+        RDocumentInterface* di = getDocumentInterface();
+        if (di!=NULL) {
+            RGraphicsView* gv = di->getLastKnownViewWithFocus();
+            if (gv!=NULL) {
+                QWidget* w = gv->getWidget();
+                if (w!=NULL) {
+                    w->setFocus(Qt::ActiveWindowFocusReason);
+                }
+            }
+        }
+    }
+    break;
+
+    case QEvent::PaletteChange:
+    case QEvent::ApplicationPaletteChange:
+    {
+        RSettings::resetDarkModeCache();
+
+        static int darkMode = -1;
+        if (darkMode!=(int)RSettings::isDarkMode()) {
+            // mode has changed from dark to light or vice versa:
+            RSettings::resetCache();
+            RGuiAction::updateIcons();
+            RGuiAction::updateToolTips();
+            notifyPaletteListeners();
+            update();
+            darkMode = (int)RSettings::isDarkMode();
+        }
+    }
+    break;
+
+    case QEvent::KeyPress:
+    {
         QKeyEvent* ke = dynamic_cast<QKeyEvent*>(e);
         if (ke!=NULL) {
 
@@ -658,18 +712,48 @@ bool RMainWindowQt::event(QEvent* e) {
             // e.g. for up / down / left / right keys
             notifyKeyListeners(ke);
 
+            // enter:
             if (ke->key()==Qt::Key_Enter || ke->key()==Qt::Key_Return) {
                 QWidget* w = QApplication::focusWidget();
                 if (w!=NULL) {
+                    // focus with view, main window or tool button:
+                    // repeat last command:
                     if (dynamic_cast<RGraphicsViewQt*>(w)!=NULL ||
                         dynamic_cast<RMainWindowQt*>(w)!=NULL ||
                         dynamic_cast<QToolButton*>(w)!=NULL) {
 
                         emit enterPressed();
+                        e->accept();
                     }
-                    QWidget* parent = w->parentWidget();
-                    if (dynamic_cast<QToolBar*>(parent)!=NULL) {
-                        emit enterPressed();
+                    else {
+                        // enter pressed in toolbar but NOT in a line edit:
+                        QWidget* parent = w->parentWidget();
+                        if (dynamic_cast<QToolBar*>(parent)!=NULL &&
+                            dynamic_cast<QLineEdit*>(w)==NULL &&
+                            dynamic_cast<QComboBox*>(w)==NULL) {
+
+                            emit enterPressed();
+                            e->accept();
+                        }
+                        else {
+                            // enter pressed in a line edit in the options toolbar:
+                            if (dynamic_cast<QLineEdit*>(w)!=NULL &&
+                                dynamic_cast<QToolBar*>(parent)!=NULL) {
+
+                                // move focus from options toolbar line edit to graphics view:
+                                RDocumentInterface* di = getDocumentInterface();
+                                if (di!=NULL) {
+                                    RGraphicsView* gv = di->getLastKnownViewWithFocus();
+                                    if (gv!=NULL) {
+                                        QWidget* gvWidget = gv->getWidget();
+                                        if (gvWidget!=NULL) {
+                                            gvWidget->setFocus(Qt::OtherFocusReason);
+                                        }
+                                    }
+                                }
+
+                            }
+                        }
                     }
                 }
             }
@@ -679,80 +763,127 @@ bool RMainWindowQt::event(QEvent* e) {
                         keyLog.clear();
                     }
                     keyLog += QChar(ke->key());
-                    //qDebug() << "keyLog" << keyLog;
-                    if (RGuiAction::triggerByShortcut(keyLog)) {
+                    //qDebug() << "keyLog:" << keyLog;
+
+                    if (mainWindowProxy && mainWindowProxy->handleOptionsShortcut(keyLog)) {
+                        // shortcut for options toolbar:
+                        keyLog.clear();
+                    }
+                    else if (RGuiAction::triggerByShortcut(keyLog)) {
+                        // shortcut for tool:
                         keyLog.clear();
                     }
                     else {
+                        // wait for additional keystrokes:
                         keyTimeOut.restart();
                     }
                 }
-                e->accept();
             }
         }
-        return true;
+    }
+    break;
+
+
+    case QEvent::IconDrag:
+    {
+        // TODO: never triggered on macOS when dragging title bar icon
+        //    QIconDragEvent* ide = dynamic_cast<QIconDragEvent*>(e);
+        //    if (ide!=NULL) {
+        //        qDebug() << "QIconDragEvent";
+        //        ide->accept();
+        //        return true;
+        //    }
     }
 
-    RSelectionChangedEvent* sce = dynamic_cast<RSelectionChangedEvent*>(e);
-    if (sce!=NULL) {
-        // selection changed: reset property editor filter to show all entities:
-        notifyPropertyListeners(getDocument(), false, RS::EntityAll);
-        notifySelectionListeners(getDocumentInterface());
-        return true;
-    }
 
-    RCoordinateEvent* coe = dynamic_cast<RCoordinateEvent*>(e);
-    if (coe!=NULL) {
-        RDocumentInterface* di = getDocumentInterface();
-        if (di!=NULL) {
-            di->coordinateEvent(*coe);
-        }
-        return true;
-    }
-
-    RTransactionEvent* te = dynamic_cast<RTransactionEvent*>(e);
-    if (te!=NULL) {
-        // combined properties might have changed (deleted entities):
-        notifyPropertyListeners(getDocument(), te->hasOnlyChanges());
-        // selection might have changed (deleted entities):
-        notifySelectionListeners(getDocumentInterface());
-        // notify transaction listeners:
-        RTransaction t = te->getTransaction();
-        notifyTransactionListeners(getDocument(), &t);
-        return true;
-    }
-
-    RPropertyEvent* pe = dynamic_cast<RPropertyEvent*>(e);
-    if (pe!=NULL) {
-        RDocumentInterface* documentInterface = getDocumentInterface();
-        if (documentInterface!=NULL) {
-            documentInterface->propertyChangeEvent(*pe);
-        }
-    }
-
-    RCloseCurrentEvent* cce = dynamic_cast<RCloseCurrentEvent*>(e);
-    if (cce!=NULL) {
-        if (mdiArea==NULL) {
+    case QEvent::User+100:
+    {
+        RSelectionChangedEvent* sce = dynamic_cast<RSelectionChangedEvent*>(e);
+        if (sce!=NULL) {
+            // selection changed: reset property editor filter to show all entities:
+            notifyPropertyListeners(getDocument(), false, RS::EntityAll);
+            notifySelectionListeners(getDocumentInterface());
             return true;
         }
+    }
+    break;
 
-        // workaround for QMdiArea bug: last window cannot be closed sometimes:
-        if (mdiArea->activeSubWindow()==NULL) {
-            QList<QMdiSubWindow *> subWindows = mdiArea->subWindowList();
-            if (subWindows.size()==1) {
-                //mdiArea->setActiveSubWindow(subWindows.at(0));
-                qDebug() << "RMainWindowQt::event: closing subwindow";
-                subWindows.at(0)->close();
+
+    case QEvent::User+201:
+    {
+        RCoordinateEvent* coe = dynamic_cast<RCoordinateEvent*>(e);
+        if (coe!=NULL) {
+            RDocumentInterface* di = getDocumentInterface();
+            if (di!=NULL) {
+                di->coordinateEvent(*coe);
             }
+            return true;
         }
-        else {
-            qDebug() << "RMainWindowQt::event: closing active subwindow";
-            mdiArea->closeActiveSubWindow();
+    }
+    break;
+
+
+    case QEvent::User+300:
+    {
+        RTransactionEvent* te = dynamic_cast<RTransactionEvent*>(e);
+        if (te!=NULL) {
+            // combined properties might have changed (deleted entities):
+            notifyPropertyListeners(getDocument(), te->hasOnlyChanges());
+            // selection might have changed (deleted entities):
+            notifySelectionListeners(getDocumentInterface());
+            // notify transaction listeners:
+            RTransaction t = te->getTransaction();
+            notifyTransactionListeners(getDocument(), &t);
+            return true;
         }
-        if (RSettings::getBoolValue("TabBar/ShowAddTabButton", false)) {
-            mdiArea->updateTabBar();
+    }
+    break;
+
+
+    case QEvent::User+400:
+    {
+        RCloseCurrentEvent* cce = dynamic_cast<RCloseCurrentEvent*>(e);
+        if (cce!=NULL) {
+            if (mdiArea==NULL) {
+                return true;
+            }
+
+            // workaround for QMdiArea bug: last window cannot be closed sometimes:
+            if (mdiArea->activeSubWindow()==NULL) {
+                QList<QMdiSubWindow *> subWindows = mdiArea->subWindowList();
+                if (subWindows.size()==1) {
+                    //mdiArea->setActiveSubWindow(subWindows.at(0));
+                    qDebug() << "RMainWindowQt::event: closing subwindow";
+                    subWindows.at(0)->close();
+                }
+            }
+            else {
+                qDebug() << "RMainWindowQt::event: closing active subwindow";
+                mdiArea->closeActiveSubWindow();
+            }
+            if (RSettings::getBoolValue("TabBar/ShowAddTabButton", false)) {
+                mdiArea->updateTabBar();
+            }
+            return true;
         }
-        return true;
+    }
+    break;
+
+
+    case QEvent::User+500:
+    {
+        RPropertyEvent* pe = dynamic_cast<RPropertyEvent*>(e);
+        if (pe!=NULL) {
+            RDocumentInterface* documentInterface = getDocumentInterface();
+            if (documentInterface!=NULL) {
+                // called when user changed a property in the property editor
+                documentInterface->propertyChangeEvent(*pe);
+            }
+            return true;
+        }
+    }
+    break;
+
     }
 
 #if QT_VERSION < 0x050000 && (defined(Q_OS_MAC) || defined(Q_OS_WIN))
