@@ -26,9 +26,11 @@
 
 #include "RArcEntity.h"
 #include "RAttributeEntity.h"
+#include "RBlock.h"
 #include "RBlockReferenceEntity.h"
 #include "RCircleEntity.h"
 #include "RColor.h"
+#include "RDocument.h"
 #include "RDimAlignedEntity.h"
 #include "RDimAngular2LEntity.h"
 #include "RDimAngular3PEntity.h"
@@ -42,6 +44,7 @@
 #include "REllipseEntity.h"
 #include "RHatchEntity.h"
 #include "RImageEntity.h"
+#include "RLayer.h"
 #include "RLeaderEntity.h"
 #include "RLineEntity.h"
 #include "RPointEntity.h"
@@ -1237,7 +1240,7 @@ void RDxfExporter::writeDimension(const RDimensionEntity& d) {
                              (const char*)RDxfExporter::escapeUnicode(text),
                              // TODO: dim style:
                              (const char*)RDxfExporter::escapeUnicode(d.getFontName()),
-                             d.getTextAngle(),
+                             d.getTextRotation(),
                              d.getLinearFactor(),
                              d.getDimscale());
     dimData.arrow1Flipped = d.isArrow1Flipped();
