@@ -121,6 +121,7 @@
 #include "REcmaEntityExportListenerAdapter.h"
 #include "REcmaEntityPickEvent.h"
 #include "REcmaEventFilter.h"
+#include "REcmaToolOptionEventFilter.h"
 #include "REcmaEventHandler.h"
 #include "REcmaExporter.h"
 #include "REcmaExportListener.h"
@@ -152,6 +153,7 @@
 #include "REcmaGraphicsView.h"
 #include "REcmaGraphicsViewImage.h"
 #include "REcmaGraphicsViewQt.h"
+#include "REcmaGraphicsViewWorker.h"
 #include "REcmaGrid.h"
 #include "REcmaGuiAction.h"
 #include "REcmaHatchData.h"
@@ -312,6 +314,7 @@
 #include "REcmaSharedPointerUcs.h"
 #include "REcmaSharedPointerView.h"
 #include "REcmaSharedPointerViewportEntity.h"
+#include "REcmaSharedPointerWipeoutEntity.h"
 #include "REcmaSharedPointerXLine.h"
 #include "REcmaSharedPointerXLineEntity.h"
 #include "REcmaSnap.h"
@@ -378,6 +381,8 @@
 #include "REcmaViewListenerAdapter.h"
 #include "REcmaViewportData.h"
 #include "REcmaViewportEntity.h"
+#include "REcmaWipeoutData.h"
+#include "REcmaWipeoutEntity.h"
 #include "REcmaWheelEvent.h"
 #include "REcmaWidget.h"
 #include "REcmaXLine.h"
@@ -731,6 +736,7 @@ RScriptHandlerEcma::RScriptHandlerEcma() : engine(NULL), debugger(NULL) {
     REcmaGraphicsView::initEcma(*engine);
     REcmaGraphicsViewImage::initEcma(*engine);
     REcmaGraphicsViewQt::initEcma(*engine);
+    REcmaGraphicsViewWorker::initEcma(*engine);
 
     REcmaMainWindow::initEcma(*engine);
     REcmaMainWindowQt::initEcma(*engine);
@@ -745,6 +751,7 @@ RScriptHandlerEcma::RScriptHandlerEcma() : engine(NULL), debugger(NULL) {
     REcmaScriptHandler::initEcma(*engine);
 
     REcmaEventFilter::initEcma(*engine);
+    REcmaToolOptionEventFilter::initEcma(*engine);
     REcmaEventHandler::initEcma(*engine);
 
     REcmaTerminateEvent::initEcma(*engine);
@@ -835,6 +842,10 @@ RScriptHandlerEcma::RScriptHandlerEcma() : engine(NULL), debugger(NULL) {
     REcmaViewportEntity::initEcma(*engine);
     REcmaSharedPointerViewportEntity::initEcma(*engine);
 
+    REcmaWipeoutData::initEcma(*engine);
+    REcmaWipeoutEntity::initEcma(*engine);
+    REcmaSharedPointerWipeoutEntity::initEcma(*engine);
+
     REcmaSplineData::initEcma(*engine);
     REcmaSplineEntity::initEcma(*engine);
     REcmaSharedPointerSplineEntity::initEcma(*engine);
