@@ -17,14 +17,18 @@
  * along with QCAD.
  */
 #include "RDocumentInterface.h"
+#include "REntity.h"
 #include "RGraphicsView.h"
 #include "RSnapPerpendicular.h"
+#include "RLine.h"
+#include "RCircle.h"
 
 QList<RVector> RSnapPerpendicular::snapEntity(
         QSharedPointer<REntity> entity,
         const RVector& point,
         const RBox& queryBox,
-        RGraphicsView& view) {
+        RGraphicsView& view,
+        QList<REntity::Id>* subEntityIds) {
 
     QList<RVector> ret;
 
@@ -33,12 +37,27 @@ QList<RVector> RSnapPerpendicular::snapEntity(
         return ret;
     }
 
-    QSharedPointer<RShape> shape = entity->getClosestShape(point, queryBox.getWidth()/2, true);
+    RObject::Id subEntityId = REntity::INVALID_ID;
+    QSharedPointer<RShape> shape = entity->getClosestShape(point, queryBox.getWidth()/2, true, &subEntityId);
     if (shape.isNull()) {
         return ret;
     }
 
-    ret.append(shape->getClosestPointOnShape(di->getLastPosition(), false));
+    if (shape->getShapeType()==RShape::Circle || shape->getShapeType()==RShape::Arc) {
+        RVector closestPoint = shape->getClosestPointOnShape(di->getLastPosition());
+        // find two perpendicular points:
+        RLine line(di->getLastPosition(), closestPoint);
+        QList<RVector> ips = line.getIntersectionPoints(*shape.data(), false);
+        ret = ips;
+    }
+    else {
+        ret.append(shape->getClosestPointOnShape(di->getLastPosition(), false));
+    }
+
+    if (subEntityIds!=NULL) {
+        subEntityIds->append(subEntityId);
+    }
+
 
     return ret;
 }
