@@ -4,19 +4,19 @@
 <context>
     <name>REntity</name>
     <message>
-        <location filename="../src/entity/RArcEntity.cpp" line="+92"/>
+        <location filename="../src/entity/RArcEntity.cpp" line="+95"/>
         <location line="+1"/>
         <location line="+1"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+79"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RDimAngularEntity.cpp" line="+109"/>
+        <location filename="../src/entity/RDimAngularEntity.cpp" line="+157"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RDimRadialEntity.cpp" line="+105"/>
+        <location filename="../src/entity/RDimRadialEntity.cpp" line="+152"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/REllipseEntity.cpp" line="+82"/>
+        <location filename="../src/entity/REllipseEntity.cpp" line="+81"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Center</source>
@@ -24,47 +24,49 @@
     </message>
     <message>
         <location line="-2"/>
+        <location line="+3"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="-2"/>
-        <location filename="../src/entity/RDimAngular2LEntity.cpp" line="+117"/>
+        <location filename="../src/entity/RDimAngular2LEntity.cpp" line="+164"/>
         <location line="+12"/>
         <location filename="../src/entity/RDimAngularEntity.cpp" line="-2"/>
         <location line="+4"/>
         <location line="+4"/>
         <location line="+4"/>
-        <location filename="../src/entity/RDimDiametricEntity.cpp" line="+107"/>
+        <location filename="../src/entity/RDimDiametricEntity.cpp" line="+154"/>
         <location line="+4"/>
-        <location filename="../src/entity/RDimLinearEntity.cpp" line="+106"/>
+        <location filename="../src/entity/RDimLinearEntity.cpp" line="+156"/>
         <location line="+4"/>
         <location line="+4"/>
-        <location filename="../src/entity/RDimOrdinateEntity.cpp" line="+107"/>
+        <location filename="../src/entity/RDimOrdinateEntity.cpp" line="+117"/>
         <location line="+5"/>
         <location line="+3"/>
         <location filename="../src/entity/RDimRadialEntity.cpp" line="-2"/>
         <location line="+4"/>
-        <location filename="../src/entity/RDimensionEntity.cpp" line="+94"/>
+        <location filename="../src/entity/RDimensionEntity.cpp" line="+121"/>
         <location line="+3"/>
         <location filename="../src/entity/REllipseEntity.cpp" line="-2"/>
         <location line="+3"/>
         <location line="+10"/>
         <location line="+3"/>
-        <location filename="../src/entity/RFaceEntity.cpp" line="+76"/>
+        <location filename="../src/entity/RFaceEntity.cpp" line="+75"/>
         <location line="+3"/>
         <location line="+3"/>
         <location line="+3"/>
-        <location filename="../src/entity/RHatchEntity.cpp" line="+93"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="+97"/>
         <location line="+3"/>
-        <location filename="../src/entity/RImageEntity.cpp" line="+94"/>
+        <location filename="../src/entity/RImageEntity.cpp" line="+84"/>
         <location line="+4"/>
         <location line="+3"/>
-        <location filename="../src/entity/RLeaderEntity.cpp" line="+78"/>
-        <location filename="../src/entity/RLineEntity.cpp" line="+84"/>
+        <location filename="../src/entity/RLeaderEntity.cpp" line="+90"/>
+        <location filename="../src/entity/RLineEntity.cpp" line="+88"/>
+        <location line="+3"/>
         <location line="+3"/>
         <location filename="../src/entity/RPointEntity.cpp" line="+72"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+108"/>
-        <location filename="../src/entity/RRayEntity.cpp" line="+79"/>
+        <location filename="../src/entity/RRayEntity.cpp" line="+78"/>
         <location line="+3"/>
         <location line="+4"/>
-        <location filename="../src/entity/RSolidEntity.cpp" line="+76"/>
+        <location filename="../src/entity/RSolidEntity.cpp" line="+75"/>
         <location line="+3"/>
         <location line="+3"/>
         <location line="+3"/>
@@ -72,20 +74,22 @@
         <location line="+3"/>
         <location line="+10"/>
         <location line="+4"/>
-        <location filename="../src/entity/RToleranceEntity.cpp" line="+77"/>
+        <location filename="../src/entity/RToleranceEntity.cpp" line="+82"/>
         <location line="+3"/>
-        <location filename="../src/entity/RTraceEntity.cpp" line="+76"/>
+        <location filename="../src/entity/RTraceEntity.cpp" line="+75"/>
         <location line="+3"/>
         <location line="+3"/>
         <location line="+3"/>
-        <location filename="../src/entity/RXLineEntity.cpp" line="+79"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+82"/>
+        <location filename="../src/entity/RXLineEntity.cpp" line="+78"/>
         <location line="+3"/>
         <location line="+4"/>
         <source>X</source>
         <translation>X</translation>
     </message>
     <message>
-        <location line="+1"/>
+        <location line="-2"/>
+        <location line="+3"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+1"/>
         <location filename="../src/entity/RDimAngular2LEntity.cpp" line="-11"/>
         <location line="+12"/>
@@ -119,7 +123,8 @@
         <location line="+4"/>
         <location line="+3"/>
         <location filename="../src/entity/RLeaderEntity.cpp" line="+1"/>
-        <location filename="../src/entity/RLineEntity.cpp" line="-2"/>
+        <location filename="../src/entity/RLineEntity.cpp" line="-5"/>
+        <location line="+3"/>
         <location line="+3"/>
         <location filename="../src/entity/RPointEntity.cpp" line="+1"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+1"/>
@@ -140,6 +145,7 @@
         <location line="+3"/>
         <location line="+3"/>
         <location line="+3"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+1"/>
         <location filename="../src/entity/RXLineEntity.cpp" line="-6"/>
         <location line="+3"/>
         <location line="+4"/>
@@ -147,7 +153,8 @@
         <translation>Y</translation>
     </message>
     <message>
-        <location line="+1"/>
+        <location line="-2"/>
+        <location line="+3"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+1"/>
         <location filename="../src/entity/RDimAngular2LEntity.cpp" line="-11"/>
         <location line="+12"/>
@@ -178,7 +185,8 @@
         <location filename="../src/entity/RHatchEntity.cpp" line="+1"/>
         <location filename="../src/entity/RImageEntity.cpp" line="-6"/>
         <location filename="../src/entity/RLeaderEntity.cpp" line="+1"/>
-        <location filename="../src/entity/RLineEntity.cpp" line="-2"/>
+        <location filename="../src/entity/RLineEntity.cpp" line="-5"/>
+        <location line="+3"/>
         <location line="+3"/>
         <location filename="../src/entity/RPointEntity.cpp" line="+1"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+1"/>
@@ -199,12 +207,20 @@
         <location line="+3"/>
         <location line="+3"/>
         <location line="+3"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+1"/>
         <location filename="../src/entity/RXLineEntity.cpp" line="-6"/>
         <location line="+3"/>
         <location line="+4"/>
         <source>Z</source>
         <translation>Z</translation>
     </message>
+    <message>
+        <location line="-2"/>
+        <location line="+1"/>
+        <location line="+1"/>
+        <source>Middle</source>
+        <translation>Middle</translation>
+    </message>
     <message>
         <location line="+1"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+1"/>
@@ -239,6 +255,8 @@
         <location line="+1"/>
         <location filename="../src/entity/REllipseEntity.cpp" line="+9"/>
         <location filename="../src/entity/RFaceEntity.cpp" line="+2"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="+3"/>
+        <location filename="../src/entity/RLeaderEntity.cpp" line="+7"/>
         <location filename="../src/entity/RLineEntity.cpp" line="+3"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+10"/>
         <location filename="../src/entity/RSolidEntity.cpp" line="+2"/>
@@ -250,6 +268,8 @@
     <message>
         <location line="+1"/>
         <location filename="../src/entity/RFaceEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RLeaderEntity.cpp" line="+1"/>
         <location filename="../src/entity/RLineEntity.cpp" line="+1"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+1"/>
         <location filename="../src/entity/RSolidEntity.cpp" line="+1"/>
@@ -266,17 +286,15 @@
     <message>
         <location line="+1"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+2"/>
-        <location filename="../src/entity/RPolylineEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RPolylineEntity.cpp" line="+3"/>
         <source>Area</source>
-        <translatorcomment>--google--
-Obszar wieloboczny
-Obszar &amp;wieloboczny
-Mierzy obszar wieloboku</translatorcomment>
-        <translation>Pole powierzchni</translation>
+        <translation>Obszar</translation>
     </message>
     <message>
         <location line="+1"/>
         <location filename="../src/entity/RCircleEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="+1"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="+1"/>
         <source>Total Area</source>
         <translation>Łączne pole powierzchni</translation>
@@ -291,9 +309,6 @@ Mierzy obszar wieloboku</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Dimension Line</source>
-        <translatorcomment>--google--
-Wskaż położenie linii wymiarowej:
-Wskaż położenie linii wymiarowej</translatorcomment>
         <translation>Linia wymiarowa</translation>
     </message>
     <message>
@@ -301,20 +316,14 @@ Wskaż położenie linii wymiarowej</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Extension Point 1</source>
-        <translatorcomment>--google--
-Punkt 1 rozszerzenia|X
-Punkt 1 rozszerzenia|Y</translatorcomment>
-        <translation>Punkt linii pomocniczej 1</translation>
+        <translation>Punkt przedłużenia 1</translation>
     </message>
     <message>
         <location line="+2"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Extension Point 2</source>
-        <translatorcomment>--google--
-Punkt 2 rozszerzenia|X
-Punkt 2 rozszerzenia|Y</translatorcomment>
-        <translation>Punkt linii pomocniczej 2</translation>
+        <translation>Punkt przedłużenia 2</translation>
     </message>
     <message>
         <location filename="../src/entity/RDimAngular2LEntity.cpp" line="-14"/>
@@ -349,9 +358,7 @@ Punkt 2 rozszerzenia|Y</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Dimension Arc</source>
-        <translatorcomment>--google--
-Wskaż położenie linii wymiarowej łuku</translatorcomment>
-        <translation>Wymiar łuku</translation>
+        <translation>Wymiar Łuk</translation>
     </message>
     <message>
         <location filename="../src/entity/RDimDiametricEntity.cpp" line="-6"/>
@@ -361,7 +368,6 @@ Wskaż położenie linii wymiarowej łuku</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Chord Point</source>
-        <translatorcomment>--google--</translatorcomment>
         <translation>Punkt cięciwy</translation>
     </message>
     <message>
@@ -369,21 +375,16 @@ Wskaż położenie linii wymiarowej łuku</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Far Chord Point</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Odległy punkt cięciwy</translation>
+        <translation>Punkt cięciwy dalekiej</translation>
     </message>
     <message>
         <location filename="../src/entity/RDimOrdinateEntity.cpp" line="-10"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RHatchEntity.cpp" line="-5"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="-11"/>
         <location line="+1"/>
         <source>Origin</source>
-        <translatorcomment>--google--
-Wskaż początek pierwszej linii rozszerzenia
-Wskaż początek drugiej linii rozszerzenia
-&amp;Usuń oryginał</translatorcomment>
-        <translation>Początek układu współrzędnych</translation>
+        <translation>Pochodzenie</translation>
     </message>
     <message>
         <location line="+2"/>
@@ -395,22 +396,19 @@ Wskaż początek drugiej linii rozszerzenia
         <location line="+1"/>
         <location line="+1"/>
         <source>Defining Point</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Definiowanie punktu</translation>
+        <translation>Określenie punktu</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Leader End Point</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Punkt końcowy odnośnika</translation>
+        <translation>Punkt końcowy lidera</translation>
     </message>
     <message>
-        <location filename="../src/entity/RDimRotatedEntity.cpp" line="+123"/>
+        <location filename="../src/entity/RDimRotatedEntity.cpp" line="+172"/>
         <source>Dimension Angle</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Kąt wymiaru</translation>
+        <translation>Wymiar Kąt</translation>
     </message>
     <message>
         <location filename="../src/entity/RDimensionEntity.cpp" line="-8"/>
@@ -426,50 +424,51 @@ Wskaż początek drugiej linii rozszerzenia
     <message>
         <location line="-1"/>
         <source>Upper Limit</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Górna odchyłka</translation>
+        <translation>Górna granica</translation>
     </message>
     <message>
         <location line="+1"/>
         <source>Lower Limit</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Dolna odchyłka</translation>
+        <translation>Dolna granica</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Definition Point</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Punkt definicji</translation>
+        <translation>Definicja Punkt</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Text Position</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Położenie tekstu</translation>
+        <translation>Pozycja tekstu</translation>
+    </message>
+    <message>
+        <location line="+3"/>
+        <source>Text Rotation</source>
+        <translation>Obracanie tekstu</translation>
     </message>
     <message>
         <location line="+3"/>
         <source>Flip First Arrow</source>
-        <translation type="unfinished"></translation>
+        <translation>Odwróć pierwszą strzałkę</translation>
     </message>
     <message>
         <location line="+1"/>
         <source>Flip Second Arrow</source>
-        <translation type="unfinished"></translation>
+        <translation>Przerzuć drugą strzałę</translation>
     </message>
     <message>
         <location line="+2"/>
         <source>Fixed Extension Line</source>
-        <translation type="unfinished"></translation>
+        <translation>Stała linia przedłużająca</translation>
     </message>
     <message>
         <location line="+1"/>
         <source>Extension Line Length</source>
-        <translation type="unfinished"></translation>
+        <translation>Długość przewodu przedłużającego</translation>
     </message>
     <message>
         <location line="+8"/>
@@ -482,12 +481,7 @@ Wskaż początek drugiej linii rozszerzenia
         <translation>Zmierzona wartość</translation>
     </message>
     <message>
-        <location line="+1"/>
-        <source>Linear Factor</source>
-        <translation>Współczynnik liniowy</translation>
-    </message>
-    <message>
-        <location line="+2"/>
+        <location line="+3"/>
         <source>Block Name</source>
         <translation>Nazwa bloku</translation>
     </message>
@@ -501,9 +495,6 @@ Wskaż początek drugiej linii rozszerzenia
         <location line="+1"/>
         <location line="+1"/>
         <source>Major Point</source>
-        <translatorcomment>--google--
-Punkt główny|X
-Punkt główny|Y</translatorcomment>
         <translation>Główny punkt</translation>
     </message>
     <message>
@@ -525,7 +516,7 @@ Punkt główny|Y</translatorcomment>
         <location line="+5"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RLineEntity.cpp" line="-9"/>
+        <location filename="../src/entity/RLineEntity.cpp" line="-12"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Start Point</source>
@@ -535,21 +526,26 @@ Punkt główny|Y</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RLineEntity.cpp" line="+1"/>
+        <location filename="../src/entity/RLineEntity.cpp" line="+4"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>End Point</source>
         <translation>Punkt końcowy</translation>
     </message>
     <message>
-        <location filename="../src/entity/RHatchEntity.cpp" line="-9"/>
+        <location filename="../src/entity/RHatchEntity.cpp" line="-10"/>
         <source>Solid</source>
         <translation>Jednolity</translation>
     </message>
+    <message>
+        <location line="+1"/>
+        <source>Winding</source>
+        <translation>Nawijanie</translation>
+    </message>
     <message>
         <location line="+1"/>
         <source>Alpha</source>
-        <translation type="unfinished"></translation>
+        <translation>Alfa</translation>
     </message>
     <message>
         <location line="+2"/>
@@ -573,14 +569,14 @@ Punkt główny|Y</translatorcomment>
         <location line="+1"/>
         <location filename="../src/entity/RImageEntity.cpp" line="+14"/>
         <location filename="../src/entity/RLineEntity.cpp" line="+2"/>
-        <location filename="../src/entity/RPolylineEntity.cpp" line="-11"/>
+        <location filename="../src/entity/RPolylineEntity.cpp" line="-13"/>
         <location filename="../src/entity/RRayEntity.cpp" line="+2"/>
         <location filename="../src/entity/RXLineEntity.cpp" line="+2"/>
         <source>Angle</source>
         <translation>Kąt</translation>
     </message>
     <message>
-        <location line="+197"/>
+        <location line="+209"/>
         <source>Yes</source>
         <translation>Tak</translation>
     </message>
@@ -590,18 +586,15 @@ Punkt główny|Y</translatorcomment>
         <translation>Nie</translation>
     </message>
     <message>
-        <location filename="../src/entity/RDimensionEntity.cpp" line="-2"/>
-        <location filename="../src/entity/RHatchEntity.cpp" line="-196"/>
-        <location filename="../src/entity/RLeaderEntity.cpp" line="+2"/>
-        <location filename="../src/entity/RToleranceEntity.cpp" line="+2"/>
+        <location line="-208"/>
         <source>Scale</source>
         <translation>Skala</translation>
     </message>
     <message>
-        <location filename="../src/entity/RHatchEntity.cpp" line="+5"/>
+        <location line="+5"/>
         <location line="+1"/>
         <location line="+1"/>
-        <location filename="../src/entity/RLeaderEntity.cpp" line="-4"/>
+        <location filename="../src/entity/RLeaderEntity.cpp" line="-10"/>
         <location line="+1"/>
         <location line="+1"/>
         <location filename="../src/entity/RPolylineEntity.cpp" line="-4"/>
@@ -611,6 +604,10 @@ Punkt główny|Y</translatorcomment>
         <location line="+1"/>
         <location line="+2"/>
         <location line="+1"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="-2"/>
+        <location line="+1"/>
+        <location line="+1"/>
+        <location line="+1"/>
         <source>Vertex</source>
         <translation>Wierzchołek</translation>
     </message>
@@ -633,25 +630,23 @@ Punkt główny|Y</translatorcomment>
         <location line="+2"/>
         <location line="+1"/>
         <source>U</source>
-        <translation type="unfinished"></translation>
+        <translation>U</translation>
     </message>
     <message>
         <location line="+2"/>
         <location line="+1"/>
         <source>V</source>
-        <translation type="unfinished"></translation>
+        <translation>V</translation>
     </message>
     <message>
         <location line="+2"/>
         <source>Width Factor</source>
-        <translatorcomment>--google--</translatorcomment>
         <translation>Współczynnik szerokości</translation>
     </message>
     <message>
         <location line="+1"/>
         <source>Height Factor</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Współczynnik wysokości</translation>
+        <translation>Czynnik wysokości</translation>
     </message>
     <message>
         <location line="+2"/>
@@ -669,27 +664,24 @@ Punkt główny|Y</translatorcomment>
         <translation>Zanikanie</translation>
     </message>
     <message>
-        <location filename="../src/entity/RLeaderEntity.cpp" line="-4"/>
+        <location filename="../src/entity/RLeaderEntity.cpp" line="-5"/>
         <source>Arrow</source>
         <translation>Strzałka</translation>
     </message>
+    <message>
+        <location line="+1"/>
+        <source>Spline</source>
+        <translation>Spline</translation>
+    </message>
     <message>
         <location line="+1"/>
         <source>Arrow Block</source>
         <translation>Blok strzałki</translation>
     </message>
-    <message>
-        <source>Clockwise</source>
-        <translation type="vanished">W prawo</translation>
-    </message>
-    <message>
-        <source>Counterclockwise</source>
-        <translation type="vanished">W lewo</translation>
-    </message>
     <message>
         <location filename="../src/entity/RPolylineEntity.cpp" line="-9"/>
         <source>Polyline Pattern</source>
-        <translation type="unfinished">Wzór linii łamanej</translation>
+        <translation>Wzór polilinii</translation>
     </message>
     <message>
         <location line="+1"/>
@@ -698,9 +690,9 @@ Punkt główny|Y</translatorcomment>
     </message>
     <message>
         <location line="+4"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+0"/>
         <source>Bulge</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Wybrzuszenie</translation>
+        <translation>Bulge</translation>
     </message>
     <message>
         <location line="+3"/>
@@ -723,7 +715,10 @@ Punkt główny|Y</translatorcomment>
         <translation>Orientacja</translation>
     </message>
     <message>
-        <location line="+6"/>
+        <location line="+9"/>
+        <location line="+1"/>
+        <location line="+1"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+2"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Size</source>
@@ -731,23 +726,26 @@ Punkt główny|Y</translatorcomment>
     </message>
     <message>
         <location line="-2"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="-2"/>
         <source>Base Angle</source>
-        <translation type="unfinished">Kąt podstawy</translation>
+        <translation>Kąt nachylenia podstawy</translation>
     </message>
     <message>
         <location line="+1"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+1"/>
         <source>Size 1</source>
-        <translation type="unfinished"></translation>
+        <translation>Wielkość 1</translation>
     </message>
     <message>
         <location line="+1"/>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+1"/>
         <source>Size 2</source>
-        <translation type="unfinished"></translation>
+        <translation>Rozmiar 2</translation>
     </message>
     <message>
         <location line="+2"/>
         <source>Global Z</source>
-        <translation type="unfinished"></translation>
+        <translation>Globalny Z</translation>
     </message>
     <message>
         <location filename="../src/entity/RFaceEntity.cpp" line="-14"/>
@@ -816,10 +814,6 @@ Punkt główny|Y</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Control Point</source>
-        <translatorcomment>--google--
-Wskaż pierwszy punkt kontrolny
-Wskaż następny punkt kontrolny lub [%1]
-Wskaż następny punkt kontrolny</translatorcomment>
         <translation>Punkt kontrolny</translation>
     </message>
     <message>
@@ -827,8 +821,7 @@ Wskaż następny punkt kontrolny</translatorcomment>
         <location line="+1"/>
         <location line="+1"/>
         <source>Fit Point</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation>Punkt dopasowania</translation>
+        <translation>Fit Point</translation>
     </message>
     <message>
         <location line="+1"/>
@@ -843,48 +836,40 @@ Wskaż następny punkt kontrolny</translatorcomment>
     <message>
         <location line="+6"/>
         <source>Use Start Tangent</source>
-        <translation type="unfinished"></translation>
+        <translation>Użyj stycznej początkowej</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Start Tangent</source>
-        <translation type="unfinished"></translation>
+        <translation>Początek stycznej</translation>
     </message>
     <message>
         <location line="+1"/>
         <source>Use End Tangent</source>
-        <translation type="unfinished"></translation>
+        <translation>Wykorzystanie stycznej końcowej</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>End Tangent</source>
-        <translation type="unfinished"></translation>
+        <translation>Styczna końcowa</translation>
     </message>
     <message>
-        <source>Simple</source>
-        <translation type="vanished">Prosta</translation>
-    </message>
-    <message>
-        <source>Contents</source>
-        <translation type="obsolete">Spis treści</translation>
-    </message>
-    <message>
-        <location filename="../src/entity/RToleranceEntity.cpp" line="-7"/>
+        <location filename="../src/entity/RToleranceEntity.cpp" line="-5"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Location</source>
-        <translation type="unfinished"></translation>
+        <translation>Lokalizacja</translation>
     </message>
     <message>
         <location line="+1"/>
         <location line="+1"/>
         <location line="+1"/>
         <source>Direction</source>
-        <translation type="unfinished"></translation>
+        <translation>Kierunek</translation>
     </message>
     <message>
         <location line="+1"/>
@@ -892,53 +877,8 @@ Wskaż następny punkt kontrolny</translatorcomment>
         <translation>Tekst</translation>
     </message>
     <message>
-        <source>Plain Text</source>
-        <translation type="vanished">Zwykły tekst</translation>
-    </message>
-    <message>
-        <source>Font Name</source>
-        <translatorcomment>--google--</translatorcomment>
-        <translation type="vanished">Nazwa czcionki</translation>
-    </message>
-    <message>
-        <source>Text Height</source>
-        <translation type="vanished">Wysokość tekstu</translation>
-    </message>
-    <message>
-        <source>Text Angle</source>
-        <translation type="vanished">Kąt tekstu</translation>
-    </message>
-    <message>
-        <source>X Scale</source>
-        <translation type="vanished">Skala X</translation>
-    </message>
-    <message>
-        <source>Bold</source>
-        <translation type="vanished">Pogrubienie</translation>
-    </message>
-    <message>
-        <source>Italic</source>
-        <translation type="vanished">Kursywa</translation>
-    </message>
-    <message>
-        <source>Line Spacing</source>
-        <translation type="vanished">Odstęp między wierszami</translation>
-    </message>
-    <message>
-        <source>Alignment</source>
-        <translation type="vanished">Wyrównanie</translation>
-    </message>
-    <message>
-        <source>Horizontal</source>
-        <translation type="vanished">Poziomo</translation>
-    </message>
-    <message>
-        <source>Vertical</source>
-        <translation type="vanished">Pionowo</translation>
-    </message>
-    <message>
-        <location filename="../src/entity/RAttributeDefinitionEntity.cpp" line="+93"/>
-        <location filename="../src/entity/RAttributeEntity.cpp" line="+97"/>
+        <location filename="../src/entity/RAttributeDefinitionEntity.cpp" line="+95"/>
+        <location filename="../src/entity/RAttributeEntity.cpp" line="+101"/>
         <source>Tag</source>
         <translation>Etykieta</translation>
     </message>
@@ -984,14 +924,27 @@ Wskaż następny punkt kontrolny</translatorcomment>
         <translation>Stały kąt</translation>
     </message>
     <message>
+        <location filename="../src/entity/RAttributeDefinitionEntity.cpp" line="-3"/>
         <location filename="../src/entity/RAttributeEntity.cpp" line="+1"/>
         <source>Invisible</source>
         <translation>Niewidoczny</translation>
     </message>
     <message>
-        <location filename="../src/entity/RDimArcLengthEntity.cpp" line="+143"/>
+        <location filename="../src/entity/RDimArcLengthEntity.cpp" line="+190"/>
         <source>Arc Symbol Type</source>
-        <translation type="unfinished"></translation>
+        <translation>Symbol łuku Typ</translation>
+    </message>
+    <message>
+        <location filename="../src/entity/RLineEntity.cpp" line="-7"/>
+        <location line="+1"/>
+        <location line="+1"/>
+        <source>Middle Point</source>
+        <translation>Middle Point</translation>
+    </message>
+    <message>
+        <location filename="../src/entity/RWipeoutEntity.cpp" line="+2"/>
+        <source>Frame</source>
+        <translation>Rama</translation>
     </message>
 </context>
 </TS>
